{
*****************************************************************************
*  Name    : W5500-DHCP.BAS                                                 *
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 14/10/2014                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}

Module W5500_DHCP

Include "W5500-Config"
Include "W5500-Consts"
Include "W5500-Regs"
Include "W5500-TX"
Include "W5500-RX"
Include "W5500-Socket"
Include "W5500-Utils"

Include "string"
Include "convert"

#if WIZ_DHCP_DEBUG = true
Include "usart"
#endif

Type TEvent = Event()

Public Dim 
    FOnUpdateEvent      As TEvent,
    FOnConflictEvent    As TEvent    

Private Const
    cDHCP_TIMEOUT       = $FFFF, 
    cDHCP_LEASE_TIME    = $FF

Private Const        //* DHCP state machine. */
    cSTATE_DHCP_INIT         = 0,
    cSTATE_DHCP_DISCOVER     = 1,
    cSTATE_DHCP_REQUEST      = 2,
    cSTATE_DHCP_LEASED       = 3,
    cSTATE_DHCP_REREQUEST    = 4,
    cSTATE_DHCP_RELEASE      = 5

    
'Structure _un_l2cval 
'    lVal As LongWord Union
'	cVal(4) As Byte Union
'End Structure


Private Dim
'    lease_time          As _un_l2cval,
    DHCP_SOCK           As Byte

Private Dim 
    TimeoutCount    As LongWord, 
    TimeoutFlag     As Boolean
    

Private Dim
    GET_SN_MASK(4)      As Byte,    // Subnet mask received from the DHCP server 
    GET_GW_IP(4)        As Byte,	// Gateway ip address received from the DHCP server 
    GET_DNS_IP(4)       As Byte,    // DNS server ip address received from the DHCP server 
    GET_SIP(4)          As Byte 	// Local ip address received from the DHCP server 

Private Dim 
    DHCP_XID            As LongWord,
    DHCP_SECS           As Word,
    DHCP_STATE          As Byte, 
    DHCP_SIP(4)         As Byte,    // DHCP server ip address is discovered 
    DHCP_REAL_SIP(4)    As Byte     // For extract my DHCP server in a few DHCP servers 
'    OLD_SIP(4)          As Byte     // Previous local ip address received from DHCP server

//* UDP Port numbers For DHCP */
Private Const
    cudpDHCP_SERVER_PORT	= 67,	//* from server to client */
    cudpDHCP_CLIENT_PORT	= 68	//* from client to server */

//* DHCP message OP code */
Private Const
    copcDHCP_BOOTREQUEST	= 1,
    copcDHCP_BOOTREPLY		= 2

Private Const
    chwtDHCP_HTYPE10MB		= 1,
    chwtDHCP_HTYPE100MB		= 2

Private Const
    DHCP_HLENETHERNET	= 6,
    DHCP_HOPS		    = 0

Private Const
    DHCP_FLAGSBROADCAST	= $8000


Private Const
    MAGIC_COOKIE		= $63825363

//* DHCP message Type */
Private Const
    cmtDHCP_DISCOVER	= 1,
    cmtDHCP_OFFER		= 2,
    cmtDHCP_REQUEST		= 3,
    cmtDHCP_DECLINE		= 4,
    cmtDHCP_ACK		    = 5,
    cmtDHCP_NAK		    = 6,
    cmtDHCP_RELEASE		= 7,
    cmtDHCP_INFORM		= 8

Private Const
	padOption                  =   0,
	subnetMask                 =   1,
	timerOffset                =   2,
	routersOnSubnet            =   3,
	timeServer                 =   4,
	nameServer                 =   5,
	dns                        =   6,
	logServer                  =   7,
	cookieServer               =   8,
	lprServer		           =   9,
	impressServer		       =   10,
	resourceLocationServer	   =   11,
	hostName                   =   12,
	bootFileSize               =   13,
	meritDumpFile              =   14,
	domainName                 =   15,
	swapServer                 =   16,
	rootPath                   =   17,
	extentionsPath		       =   18,
	IPforwarding               =   19,
	nonLocalSourceRouting      =   20,
	policyFilter               =   21,
	maxDgramReasmSize          =   22,
	defaultIPTTL               =   23,
	pathMTUagingTimeout        =   24,
	pathMTUplateauTable        =   25,
	ifMTU                      =   26,
	allSubnetsLocal            =   27,
	broadcastAddr              =   28,
	performMaskDiscovery       =   29,
	maskSupplier               =   30,
	performRouterDiscovery     =   31,
	routerSolicitationAddr     =   32,
	staticRoute                =   33,
	trailerEncapsulation       =   34,
	arpCacheTimeout            =   35,
	ethernetEncapsulation      =   36,
	tcpDefaultTTL              =   37,
	tcpKeepaliveInterval       =   38,
	tcpKeepaliveGarbage        =   39,
	nisDomainName              =   40,
	nisServers                 =   41,
	ntpServers                 =   42,
	vendorSpecificInfo         =   43,
	netBIOSnameServer          =   44,
	netBIOSdgramDistServer     =   45,
	netBIOSnodeType            =   46,
	netBIOSscope               =   47,
	xFontServer                =   48,
	xDisplayManager            =   49,
	dhcpRequestedIPaddr	       =   50,
	dhcpIPaddrLeaseTime        =   51,
	dhcpOptionOverload	       =   52,
	dhcpMessageType            =   53,
	dhcpServerIdentifier       =   54,
	dhcpParamRequest           =   55,
	dhcpMsg                    =   56,
	dhcpMaxMsgSize             =   57,
	dhcpT1value                =   58,
	dhcpT2value                =   59,
	dhcpClassIdentifier        =   60,
	dhcpClientIdentifier       =   61,
	endOption                  =   255
              

Private Structure RIP_MSG
	Op         As Byte 
	HType      As Byte 
	HLen       As Byte
	Hops       As Byte
	XID        As LongWord
	Secs       As Word
	Flags      As Word
	CIAddr(4)  As Byte
	YIAddr(4)  As Byte
	SIAddr(4)  As Byte
	GIAddr(4)  As Byte
	CHAddr(16) As Byte
	sName(64)  As Byte
	File(128)  As Byte
	OPT(312)   As Byte
End Structure

Private Dim 
    pRIPMSG             As RIP_MSG


{
****************************************************************************
* Name    : SetDHCPNetwork                                                 *
* Purpose : After DHCP operations set new net parms.                       *
****************************************************************************
} 
Private Sub SetDHCPNetwork(Static As Boolean = false)
	WriteCommonReg(rMR, $80)
    // modificato 27/2/2015
    ReadCommonReg(rMR)

	If Static Then
        // Restore static parameters when DHCP fails
        CArrCpy(Ip_Addr,  cIp_Addr,  4)
        CArrCpy(Sub_Mask, cSub_Mask, 4)
        CArrCpy(Gtw_Addr, cGtw_Addr, 4)
        CArrCpy(Dns_Addr, cDns_Addr, 4)
        SetGAR(@Gtw_Addr)
	    SetSAR(cMac_Addr)
        SetSUBR(@Sub_Mask)
        SetSIPR(@Ip_Addr)
    Else
        SetGAR(@GET_GW_IP)      // Assegno lindirizzo del Gateway   
        SetSAR(cMac_Addr)       // Assegno il MAC address
        SetSUBR(@GET_SN_MASK)   // Imposto il Subnet Mask Address 
        SetSIPR(@GET_SIP)       // Assegno lindirizzo IP  
    End If
    SysMemInit()    
	DelayMS(2000)
End Sub




{
****************************************************************************
* Name    : InitDHCPcch                                                    *
* Purpose : Initialize W5500 and open a socket                             *
****************************************************************************
} 
Private Function InitDHCPcch() As Byte
    Dim ret As Byte 
    ret = 0
    result = 0
	DHCP_XID = $12345678
	
	SetMem(@GET_SIP, 0, 4)
    SetMem(@GET_GW_IP, 0, 4)
	SetMem(@GET_SN_MASK, 0, 4)
	WriteCommonReg(rMR, $80)
	SetSIPR(@GET_SIP)
	SetSAR(cMac_Addr)  

	SysMemInit()

	If Not Socket(DHCP_SOCK, cSn_MR_UDP, cudpDHCP_CLIENT_PORT, $00) Then
#if WIZ_DHCP_DEBUG = true	
		USART.Write("WIZDHCP: fail..",#13,#10)
#endif
		ret = 0
	Else
#if WIZ_DHCP_DEBUG = true	
		USART.Write("WIZDHCP: ok..",#13,#10)
#endif
		ret = 1
    End If
	result = ret
End Function

{
****************************************************************************
* Name    : InitDHCPClient                                                 *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub InitDHCPClient(ByRef ip_update As Byte)
	If ip_update = 0 Then
        SetDHCPNetwork()
	Else		
        ip_update = 1
    End If
	InitDHCPcch()
End Sub

Dim OptCount As Byte

{
****************************************************************************
* Name    : FillOPT                                                        *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub FillOPT(b As Byte)
    pRIPMSG.OPT(OptCount) = b    
    Inc(OptCount)
End Sub

Private Sub FillOPT(pArrAddr As Word, len As Byte)
    Save(FSR2)
    FSR2 = pArrAddr
    While len<>0
        pRIPMSG.OPT(OptCount) = POSTINC2    
        Inc(OptCount)
        Dec(len)
    End While
    Restore
End Sub

#if WIZ_DHCP_DEBUG=true
Private Sub FillOptDump()
   Dim OptLen As Word 
   Dim c As Byte 
   Dim idx As Word
   OptLen = OptCount
   c = 0
   Idx = 0
   While OptLen <> 0
      USART.Write(HexToStr(pRIPMSG.OPT(idx),2))
      Inc(idx)
      If c = 7 Then
         USART.Write(13,10)
         c = 0
      Else
         USART.Write(",")
         Inc(c)
      End If   
   End While
   If c <> 0 Then
      USART.Write(13,10)
   End If
End Sub
#endif

{
****************************************************************************
* Name    : ParseDhcpMsg                                                   *
* Purpose :                                                                *
****************************************************************************
} 
Private Function ParseDhcpMsg( _length As Word) As Byte
	Dim svr_addr(6) As Byte
	Dim svr_port As Word
	Dim len As Word
	Dim iOpt As Word
	
	Dim typ As Byte
	Dim vOp As Byte
	Dim opt_len As Word
    Dim nbns(4) As Byte

#if WIZ_DHCP_DEBUG = true
	USART.Write("WIZDHCP: ParseDhcpMsg ",#13,#10)
#endif
    Clear(pRIPMSG)
	len = RecvFrom(DHCP_SOCK, @pRIPMSG, _length, svr_addr, svr_port)
	If pRIPMSG.Op <> copcDHCP_BOOTREPLY Then
#if WIZ_DHCP_DEBUG = true
		USART.Write("WIZDHCP: NO DHCP MSG",#13,#10)
#endif
	Else
		If svr_port = cudpDHCP_SERVER_PORT Then           
			If CArrCmp(pRIPMSG.CHAddr, cMac_Addr, 6) <> 0 Or pRIPMSG.XID <> DHCP_XID Then
#if WIZ_DHCP_DEBUG = true
				USART.Write("WIZDHCP: No My DHCP Message. This message is ignored.",#13,#10)
#endif
				result = 0
				Exit 
            End If

            If ArrToLongWord(@DHCP_SIP)<>0 And 
               ArrToLongWord(@DHCP_REAL_SIP) <> ArrToLongWord(@svr_addr) And 
               ArrToLongWord(@DHCP_REAL_SIP) <> ArrToLongWord(@svr_addr) Then
#if WIZ_DHCP_DEBUG = true
			    USART.Write("WIZDHCP: Another DHCP sever send a response message. This is ignored.",#13,#10)
#endif
                result = 0
                Exit
            End If

			
    		ArrCpy(@GET_SIP,@pRIPMSG.YIAddr,4)

            iOpt = 4 // Salto MAGIC CODE
#if WIZ_DHCP_DEBUG = true
            USART.Write("WIZDHCP: dhcpMessageOP ",DecToStr(pRIPMSG.Op),#13,#10)
#endif
            While true           
                vOp = pRIPMSG.OPT(iopt)
                Inc(iopt)
                Opt_Len = pRIPMSG.OPT(iopt)
                Inc(iopt)
                Select vOp
    				Case dhcpMessageType
    				    typ = pRIPMSG.OPT(iopt)
#if WIZ_DHCP_DEBUG = true
                        USART.Write(#13,#10,"   dhcpMessageType ",DecToStr(typ),#13,#10)                    
#endif
                    Case endOption
#if WIZ_DHCP_DEBUG = true
                        USART.Write(" ",#13,#10)
#endif
                        result = typ
                        Exit     
                    Case subnetMask
                        ArrCpy(@GET_SN_MASK, @pRIPMSG.OPT(iopt), Opt_Len)
#if WIZ_DHCP_DEBUG = true
                        USART.Write("   subnetMask ",DecToStr(GET_SN_MASK(0)),":",DecToStr(GET_SN_MASK(1)),":",DecToStr(GET_SN_MASK(2)),":",DecToStr(GET_SN_MASK(3)))
#endif
                    Case routersOnSubnet
                        ArrCpy(@GET_GW_IP, @pRIPMSG.OPT(iopt), Opt_Len)	
#if WIZ_DHCP_DEBUG = true
                        USART.Write("   routersOnSubnet ",DecToStr(GET_GW_IP(0)),":",DecToStr(GET_GW_IP(1)),":",DecToStr(GET_GW_IP(2)),":",DecToStr(GET_GW_IP(3)))                    
#endif
                    Case dns
					    ArrCpy(@GET_DNS_IP, @pRIPMSG.OPT(iopt), Opt_Len)
					    ArrCpy(@Dns_Addr, @pRIPMSG.OPT(iopt), opt_len)
#if WIZ_DHCP_DEBUG = true
                        USART.Write("   dns ",DecToStr(GET_DNS_IP(0)),":",DecToStr(GET_DNS_IP(1)),":",DecToStr(GET_DNS_IP(2)),":",DecToStr(GET_DNS_IP(3)))
#endif
                    Case domainName
					   ' memcpy(domain, @pRIPMSG.OPT(iopt), Opt_Len)
                       ' USART.Write(" domainName:",domain,#13,#10)                        
                    Case netBIOSnameServer
					    ArrCpy(@nbns, @pRIPMSG.OPT(iopt), Opt_Len)
#if WIZ_DHCP_DEBUG = true
                       USART.Write("   nbns ",DecToStr(nbns(0)),":",DecToStr(nbns(1)),":",DecToStr(nbns(2)),":",DecToStr(nbns(3)))
#endif
                    Case dhcpIPaddrLeaseTime
'                        ArrCpy(lease_time.lVal, @pRIPMSG.OPT(iopt))    
'#if WIZ_DHCP_DEBUG = true
'                        USART.Write("   lease_time ",DecToStr(lease_time.cVal(0)),":",DecToStr(lease_time.cVal(1)),":",DecToStr(lease_time.cVal(2)),":",DecToStr(lease_time.cVal(3)))
'#endif
				    Case dhcpServerIdentifier 
    					If  ( ArrToLongWord(@DHCP_SIP) = 0 ) Or 
    					    ( ArrToLongWord(@DHCP_REAL_SIP) = ArrToLongWord(@svr_addr)) Or
    					    ( ArrToLongWord(@DHCP_SIP) = ArrToLongWord(@svr_addr)) Then
    						ArrCpy(@DHCP_SIP, @pRIPMSG.OPT(iopt), Opt_Len)
    						ArrCpy(@DHCP_REAL_SIP,@svr_addr,4)	// Copy the real ip address of my DHCP server
#if WIZ_DHCP_DEBUG = true
                            USART.Write("   dhcpServerIdentifier ",DecToStr(DHCP_REAL_SIP(0)),":",DecToStr(DHCP_REAL_SIP(1)),":",DecToStr(DHCP_REAL_SIP(2)),":",DecToStr(DHCP_REAL_SIP(3)))
#endif
    					Else
    					End If                               
                End Select
#if WIZ_DHCP_DEBUG = true
                USART.Write(#13,#10)
#endif
                Inc(iopt,opt_len)    
            End While            
#if WIZ_DHCP_DEBUG = true
            USART.Write(" ",#13,#10)
#endif
		End If
	End If 
	result = 0 
End Function


{
****************************************************************************
* Name    : CheckDhcpTimeout                                               *
* Purpose :                                                                *
****************************************************************************
} 
Private Function CheckDhcpTimeout() As Boolean
    If TimeoutCount > 0 Then 
        Dec(TimeoutCount)
        TimeoutFlag = false
    Else
        TimeoutFlag = true
    End If
    result = TimeoutFlag
End Function

{
****************************************************************************
* Name    : ResetDhcpTimeout                                               *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub ResetDhcpTimeout()
    TimeoutFlag = false
    TimeoutCount = cDHCP_TIMEOUT    
End Sub

{
****************************************************************************
* Name    : FillOPTwHostName                                               *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub FillOPTwHostName()
    Dim i As Byte
	// host name
	FillOPT(hostName)
	FillOPT(Byte(Length(HOST_NAME)+3))
	For i = 0 To Length(HOST_NAME) - 1
	   FillOPT(Byte(HOST_NAME(i)))
    Next   
End Sub

Private Sub FillMagicCookie()
    FillOPT(Byte((MAGIC_COOKIE >> 24) And $FF))
    FillOPT(Byte((MAGIC_COOKIE >> 16) And $FF))
    FillOPT(Byte((MAGIC_COOKIE >> 8) And $FF))
    FillOPT(Byte(MAGIC_COOKIE And $FF))
End Sub

{
****************************************************************************
* Name    : SendDhcpDiscover                                               *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub SendDhcpDiscover()
	Dim ip(4) As Byte
	Dim i As Word

	SetMem(@DHCP_SIP, 0, 4)
    SetMem(@DHCP_REAL_SIP, 0, 4)
	SetMem(@ip, $FF, 4)
    SetDestNode(DHCP_SOCK, @ip, cudpDHCP_SERVER_PORT)  
    // copy data
    
    Clear(pRIPMSG)
    pRIPMSG.Op = copcDHCP_BOOTREQUEST
	pRIPMSG.HType = chwtDHCP_HTYPE10MB
	pRIPMSG.HLen = DHCP_HLENETHERNET
	pRIPMSG.Hops = DHCP_HOPS
	pRIPMSG.XID = DHCP_XID
	pRIPMSG.Secs = DHCP_SECS
	pRIPMSG.Flags = DHCP_FLAGSBROADCAST
	CArrCpy(pRIPMSG.CHAddr, cMac_Addr, 6) 
        
    OptCount = 0
    FillMagicCookie()
    
	FillOPT(dhcpMessageType)
	FillOPT($01)
	FillOPT(cmtDHCP_DISCOVER)

	// Client identifier
	FillOPT(dhcpClientIdentifier)
	FillOPT($07)
	FillOPT($01)
	FillOPT(@cMac_Addr,6)
	
	// host name
	FillOPT(hostName)
	FillOPT(Byte(Length(HOST_NAME)+3))
	For i = 0 To Length(HOST_NAME) - 1
	   FillOPT(Byte(HOST_NAME(i)))
    Next   
	
	FillOPT(cMac_Addr(3))
	FillOPT(cMac_Addr(4))
    FillOPT(cMac_Addr(5))
	
	FillOPT(dhcpParamRequest)
	FillOPT($06)
	FillOPT(subnetMask)
	FillOPT(routersOnSubnet)
	FillOPT(dns)
	FillOPT(domainName)
	FillOPT(dhcpT1value)
	FillOPT(dhcpT2value)
	FillOPT(endOption)


'#if WIZ_DHCP_DEBUG = true
'    FillOptDump()
'#endif
#if WIZ_DHCP_DEBUG = true
    USART.Write("WIZDHCP: SizeOf(RIP_MSG)=",DecToStr(SizeOf(RIP_MSG)),13,10)
#endif
    ASyncTXBufferInit(DHCP_SOCK)
    PutInTXBuffer(DHCP_SOCK, @pRIPMSG, SizeOf(RIP_MSG), false)   
    ASyncTXBufferSend(DHCP_SOCK)
        
    'WriteSocketReg(DHCP_SOCK, rSn_IR, cSn_IR_SENDOK)
		
#if WIZ_DHCP_DEBUG = true
	USART.Write("WIZDHCP: sent DHCP_DISCOVER",#13,#10)
#endif
End Sub


{
****************************************************************************
* Name    : SendDhcpRequest                                                *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub SendDhcpRequest()
	Dim ip(4) As Byte
	
	//* send broadcasting packet */
	If(DHCP_STATE < cSTATE_DHCP_LEASED) Then
		SetMem(@ip, $FF, 4)
	Else
        ArrCpy(@ip, @DHCP_SIP, 4)
    End If
	
	SetDestNode(DHCP_SOCK, @ip, cudpDHCP_SERVER_PORT)
	
    Clear(pRIPMSG)
    pRIPMSG.Op = copcDHCP_BOOTREQUEST
	pRIPMSG.HType = chwtDHCP_HTYPE10MB
	pRIPMSG.HLen = DHCP_HLENETHERNET
	pRIPMSG.Hops = DHCP_HOPS
	pRIPMSG.XID = DHCP_XID
	pRIPMSG.Secs = DHCP_SECS

	If(DHCP_STATE < cSTATE_DHCP_LEASED) Then
	   pRIPMSG.Flags = DHCP_FLAGSBROADCAST
	Else
	   pRIPMSG.Flags = 0
	   ArrCpy(@pRIPMSG.CIAddr, @GET_SIP, 4)
	End If		

    CArrCpy(pRIPMSG.CHAddr, cMac_Addr, 6) 
    OptCount = 0
    FillMagicCookie()
    
	FillOPT(dhcpMessageType)
	FillOPT($01)
	FillOPT(cmtDHCP_REQUEST)

 	// Client identifier
	FillOPT(dhcpClientIdentifier)
	FillOPT($07)
	FillOPT($01)
	FillOPT(@cMac_Addr, 6)
	
	If(DHCP_STATE < cSTATE_DHCP_LEASED) Then
		FillOPT(dhcpRequestedIPaddr)
		FillOPT($04)
		FillOPT(@GET_SIP, 4)
	
		FillOPT(dhcpServerIdentifier)
		FillOPT($04)
		FillOPT(@DHCP_SIP, 4)
    End If

	
    FillOPTwHostName()	
    
	FillOPT(cMac_Addr(3))
	FillOPT(cMac_Addr(4))
	FillOPT(cMac_Addr(5))
	
	FillOPT(dhcpParamRequest)
	FillOPT($08)
	FillOPT(subnetMask)
	FillOPT(routersOnSubnet)
	FillOPT(dns)
	FillOPT(domainName)
	FillOPT(dhcpT1value)
	FillOPT(dhcpT2value)
	FillOPT(performRouterDiscovery)
	FillOPT(staticRoute)
	FillOPT(endOption)

#if WIZ_DHCP_DEBUG = true
    USART.Write("WIZDHCP: SizeOf(RIP_MSG)=",DecToStr(SizeOf(RIP_MSG)),13,10)
#endif


    ASyncTXBufferInit(DHCP_SOCK)
    PutInTXBuffer(DHCP_SOCK, @pRIPMSG, SizeOf(RIP_MSG), false)    
    ASyncTXBufferSend(DHCP_SOCK)
   ' WriteSocketReg(DHCP_SOCK, rSn_IR, cSn_IR_SENDOK)
		
#if WIZ_DHCP_DEBUG = true
	USART.Write("WIZDHCP: sent DHCP_REQUEST",#13,#10)
#endif
End Sub

{
****************************************************************************
* Name    : SendDhcpReleaseDecline                                         *
* Purpose : 0 : RELEASE, Not Zero : DECLINE                                *
****************************************************************************
} 
Private Sub SendDhcpReleaseDecline(msgtype As Byte)
	Dim ip(4) As Byte
	
	Clear(pRIPMSG)

	pRIPMSG.Op     = copcDHCP_BOOTREQUEST
	pRIPMSG.HType  = chwtDHCP_HTYPE10MB
	pRIPMSG.HLen   = DHCP_HLENETHERNET
	pRIPMSG.Hops   = DHCP_HOPS
	pRIPMSG.XID    = DHCP_XID
	pRIPMSG.Secs   = DHCP_SECS
	pRIPMSG.Flags  = 0	            //DHCP_FLAGSBROADCAST;

	CArrCpy(pRIPMSG.CHAddr, cMac_Addr, 6)

    OptCount = 0
    FillMagicCookie()
    
	//* Option Request Param. */
	FillOPT(dhcpMessageType)
	FillOPT($01)
	FillOPT(cmtDHCP_REQUEST)

	FillOPT(dhcpClientIdentifier)
	FillOPT($07)
	FillOPT($01)
	FillOPT(@cMac_Addr, 6)

	FillOPT(dhcpServerIdentifier)
	FillOPT($04)
	FillOPT(@DHCP_SIP, 4)

	If (msgtype<>0 ) Then   
		FillOPT(dhcpRequestedIPaddr)
		FillOPT($04)
		FillOPT(@GET_SIP, 4)
		FillOPT(endOption)
#if WIZ_DHCP_DEBUG = true
		USART.Write("WIZDHCP: sent DHCP_DECLINE",#13,#10)
#endif
	Else
		FillOPT(endOption)
#if WIZ_DHCP_DEBUG = true
		USART.Write("WIZDHCP: sent DHCP_RELEASE",#13,#10)
#endif
    End If
	
	If ( msgtype = 0 ) Then
        ArrCpy(@ip, @DHCP_SIP, 4)
	Else
	    SetMem(@ip, $FF, 4)
    End If

#if WIZ_DHCP_DEBUG = true
    USART.Write("WIZDHCP: SizeOf(RIP_MSG)=",DecToStr(SizeOf(RIP_MSG)),13,10)
#endif

    SendTo(DHCP_SOCK, @pRIPMSG, SizeOf(RIP_MSG), ip, cudpDHCP_SERVER_PORT)    
End Sub

{
****************************************************************************
* Name    : CheckLeasedIP                                                  *
* Purpose :                                                                *
****************************************************************************
} 
Private Function CheckLeasedIP() As Boolean
	Dim a As Word
	
#if WIZ_DHCP_DEBUG = true
    USART.Write("WIZDHCP: <Check the IP Conflict : ")
#endif
	                       // sendto is complete. that means there is a node which has a same IP.

	a=0                    // Skip checking IP Confilct ; 
                           // W5500 reply itself to ARP requset with self-IP in non-switching network environment.
	If ( a > 0) Then
#if WIZ_DHCP_DEBUG = true
		USART.Write(" Conflict>",#13,#10)
#endif
		FOnConflictEvent
        SendDhcpReleaseDecline(1)
        result = false
    End If
#if WIZ_DHCP_DEBUG = true
	USART.Write(" No Conflict>",#13,#10)
#endif
	result = true
End Function


{
****************************************************************************
* Name    : CheckDhcpState                                                 *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub CheckDhcpState()
    Dim len As Word
	Dim typ As Byte
	
	typ = 0

	If( DHCP_SOCK < MAX_SOCK_NUM And ReadSocketReg( DHCP_SOCK, rSn_SR ) <> cSOCK_CLOSED) Then
    	len = ReadSocketWord(DHCP_SOCK, rSn_RX_RSR0)
		If len  > 0 Then
			typ = ParseDhcpMsg(len)
        End If
	ElseIf Socket(DHCP_SOCK, cSn_MR_UDP, cudpDHCP_CLIENT_PORT, $0) Then
#if WIZ_DHCP_DEBUG = true
		USART.Write("WIZDHCP: Fail to create the DHCPC_SOCK " , DecToStr(DHCP_SOCK),#13,#10)
#endif
    End If
	
		
	Select DHCP_STATE
	Case cSTATE_DHCP_DISCOVER 
		If typ = cmtDHCP_OFFER Then
			ResetDhcpTimeout()
			SendDhcpRequest()
			DHCP_STATE = cSTATE_DHCP_REQUEST
#if WIZ_DHCP_DEBUG = true
			USART.Write("WIZDHCP: STATE_DHCP_REQUEST",#13,#10)
#endif
		Else
            If CheckDhcpTimeout() Then Exit End If
        End If
	Case cSTATE_DHCP_REQUEST 
		If Typ = cmtDHCP_ACK Then
			ResetDhcpTimeout()
			If CheckLeasedIP() Then
				SetDHCPNetwork()
				DHCP_STATE = cSTATE_DHCP_LEASED
#if WIZ_DHCP_DEBUG = true
				USART.Write("WIZDHCP: STATE_DHCP_LEASED",#13,#10)
#endif
			    FOnUpdateEvent
			Else 
				DHCP_STATE = cSTATE_DHCP_DISCOVER
#if WIZ_DHCP_DEBUG = true
				USART.Write("WIZDHCP: STATE_DHCP_DISCOVER",#13,#10)
#endif
            End If
		ElseIf Typ = cmtDHCP_NAK Then
			ResetDhcpTimeout()
			DHCP_STATE = cSTATE_DHCP_DISCOVER
#if WIZ_DHCP_DEBUG = true
			USART.Write("WIZDHCP: STATE_DHCP_DISCOVER",#13,#10)
#endif
		Else 
            If CheckDhcpTimeout() Then Exit End If
        End If
	Case cSTATE_DHCP_LEASED
'        If ((lease_time.lVal <> $ffffffff) And ((lease_time.lVal/2) < cDHCP_LEASE_TIME)) Then
'			typ = 0
'			ArrCpy(OLD_SIP,GET_SIP,4)
'			Inc(DHCP_XID)
'			SendDhcpRequest()
'			DHCP_STATE = cSTATE_DHCP_REREQUEST
'#if WIZ_DHCP_DEBUG = true
'			USART.Write("WIZDHCP: STATE_DHCP_REREQUEST",#13,#10)
'#endif
'			ResetDhcpTimeout()
'        End If
	Case cSTATE_DHCP_REREQUEST 
'		If Typ = cmtDHCP_ACK Then 
'			If ArrCmp(OLD_SIP,GET_SIP,4) <> 0 Then	
'			    FOnUpdateEvent
'#if WIZ_DHCP_DEBUG = true
'				USART.Write("WIZDHCP: The IP address from the DHCP server is updated.",#13,#10)
'#endif
'			Else
'#if WIZ_DHCP_DEBUG = true
'				USART.Write("WIZDHCP: STATE_DHCP_LEASED same IP",#13,#10)
'#endif
'            End If
'			ResetDhcpTimeout()
'			DHCP_STATE = cSTATE_DHCP_LEASED
'		ElseIf Typ = cmtDHCP_NAK Then
'            ResetDhcpTimeout()
'			DHCP_STATE = cSTATE_DHCP_DISCOVER
'#if WIZ_DHCP_DEBUG = true
'			USART.Write("WIZDHCP: STATE_DHCP_DISCOVER",#13,#10)
'#endif
'		Else 
'            If CheckDhcpTimeout() Then Exit End If
'		End If

	Case cSTATE_DHCP_RELEASE 
	Else
	End Select
End Sub

{
****************************************************************************
* Name    : GetDhcpConfig                                                  *
* Purpose :                                                                *
****************************************************************************
} 
Private Function GetDhcpConfig() As Boolean
#if WIZ_DHCP_DEBUG = true
	USART.Write("WIZDHCP: DHCP SetIP..",#13,#10)
#endif
	SendDhcpDiscover()
	DHCP_STATE = cSTATE_DHCP_DISCOVER
	ResetDhcpTimeout()
    result = false
	While (DHCP_STATE <> cSTATE_DHCP_LEASED)
    	If CheckDhcpTimeout Then
			result = false
			Exit 
		End If
		CheckDhcpState()
    End While
	result = true
End Function

Private Dim isUpdated As Byte

Public Sub SetEvents(pOnUpdateEvent As TEvent = 0, pOnConflictEvent As TEvent = 0 )
   FOnUpdateEvent   = pOnUpdateEvent
   FOnConflictEvent = pOnConflictEvent
End Sub

{
****************************************************************************
* Name    : DHCPTask                                                       *
* Purpose :                                                                *
****************************************************************************
} 
Public Function DHCPTask(RetryCount As Byte = 3, Sock As Byte = 0) As Boolean
    result = false
    DHCP_SOCK = Sock
	InitDHCPClient(isUpdated)
    DelayMS(1000)
	While RetryCount > 0 
#if WIZ_DHCP_DEBUG = true
        USART.Write("WIZDHCP: Searching DHCP Server...",#13,#10)
#endif
        Dec(RetryCount)      
		If GetDhcpConfig() Then
#if WIZ_DHCP_DEBUG = true
    		USART.Write("WIZDHCP: Done.",#13,#10)
#endif
            result = true
            Exit 
        Else
#if WIZ_DHCP_DEBUG = true
    		USART.Write("WIZDHCP: Timeout",#13,#10) 
#endif
            DelayMS(5000) 
        End If
    End While
    SetDHCPNetwork(true)
End Function

isUpdated           = 0
TimeoutCount        = cDHCP_TIMEOUT
TimeoutFlag         = false

SetMem(@GET_SN_MASK, 0, 4)   // Subnet mask received from the DHCP server
SetMem(@GET_GW_IP, 0, 4)     // Gateway ip address received from the DHCP server
SetMem(@GET_DNS_IP, 0, 4)    // DNS server ip address received from the DHCP server
SetMem(@GET_SIP, 0, 4)       // Local ip address received from the DHCP server 
SetMem(@DHCP_SIP, 0, 4)      // DHCP server ip address is discovered 
SetMem(@DHCP_REAL_SIP, 0, 4) // For extract my DHCP server in a few DHCP servers 
'setmem(@OLD_SIP, 0, 4)       // Previous local ip address received from DHCP server  

DHCP_STATE      = cSTATE_DHCP_INIT
DHCP_SECS       = $0 
DHCP_SOCK       = 0

