{
*****************************************************************************
*  Name    : W5500-DNS.BAS                                                  *
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 23/10/2014                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Module W5500_DNS

Include "W5500-Consts"
Include "W5500-Regs"
Include "W5500-Socket"
Include "W5500-TX"
Include "W5500-RX"
Include "W5500-Utils"
Include "W5500-DNSConsts"
Include "usart"
Include "convert"
Include "string"


Public Structure WORDVAL
   Val      As  Word
   Byt(2)   As  Byte Union
End Structure

Public Structure DWORDVAL
   Val      As LongWord
   Wrd(2)   As Word Union
   Byt(4)   As Byte Union
End Structure


Structure DNS_HEADER
   TrxID As WORDVAL
   Flags As WORDVAL
   Qustn As WORDVAL
   Answr As WORDVAL
   Athrt As WORDVAL
   Addtn As WORDVAL
End Structure


Structure DNS_ANSW_HDR
   rName     As WORDVAL
   rType     As WORDVAL
   rClass    As WORDVAL
   rTTL      As DWORDVAL
   rLen      As WORDVAL
End Structure

Public Type Ip_Addr = DWORDVAL

Public Structure MAC_ADDR
   Val(6) As Byte
End Structure

Public Structure DNS_INFO
   IPAddr As Ip_Addr
   Status As Byte
   Flags  As WORDVAL
End Structure


Private Dim 
    SentTrxID       As WORDVAL,
    qType           As Byte,
    qClass          As Byte

Private Dim DNSBuff(512)    As Byte

Private Dim 
    TimeoutCount    As LongWord, 
    TimeoutFlag     As Boolean, 
    qString         As String(60)


'Private Type TROMAddress = Word

{
****************************************************************************
* Name    : CheckDNSTimeout                                               *
* Purpose :                                                                *
****************************************************************************
} 
Private Function CheckDNSTimeout() As Boolean
    If TimeoutCount > 0 Then 
        Dec(TimeoutCount)
        TimeoutFlag = false
    Else
        TimeoutFlag = true
    End If
    result = TimeoutFlag
End Function

{
****************************************************************************
* Name    : ResetDNSTimeout                                               *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub ResetDNSTimeout()
    TimeoutFlag = false
    TimeoutCount = cDNS_TIMEOUT    
End Sub

{
****************************************************************************
* Name    : DNSPutString                                                   *
* Purpose :                                                                *
****************************************************************************
}
Private Sub DNSPutString()
    Dim len As Byte
    Dim idx As Byte
    Dim c As Byte
    Dim p As Byte
    len = Length(qString)
    idx = 0
    p = 0
    While len>0 
        Dec(len)
        c = Byte(qString(idx))    
        If c <> Byte(".") And c <> Byte(",") And c <> Byte("/") And c <> Byte(">") Then
            DNSBuff(p) = c
            Inc(p)
        Else
#if WIZ_DNS_DEBUG = true	
            USART.Write("WIZDNS: DNSPutString Len=",DecToStr(p),#13,#10)
#endif
            PutInTXBuffer(DNS_SOCK,p)
            PutInTXBuffer(DNS_SOCK,@DNSBuff,p,false)
            p = 0
        End If
        Inc(idx) 
    End While
    If p<>0 Then
#if WIZ_DNS_DEBUG = true	
        USART.Write("WIZDNS: DNSPutString Len=",DecToStr(p),#13,#10)
#endif
        PutInTXBuffer(DNS_SOCK,p)
        PutInTXBuffer(DNS_SOCK,@DNSBuff,p,false)    
    End If
    PutInTXBuffer(DNS_SOCK,0)
End Sub



Dim DNSBuffIdx As Word
{
****************************************************************************
* Name    : DNSGetByte                                                     *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub DNSGetByte(ByRef bRet As Byte)
    bRet = DNSBuff(DNSBuffIdx)
    Inc(DNSBuffIdx)    
End Sub

{
****************************************************************************
* Name    : DNSSpoolString                                                 *
* Purpose :                                                                *
****************************************************************************
} 
Sub DNSSpoolString()
    Dim i As Byte
    Dim c As Byte
    While true
        DNSGetByte(i)
        If i = 0 Then
            Exit
        EndIf
        While i>0
            DNSGetByte(c)
            Dec(i)
        End While
    Wend
End Sub

{
****************************************************************************
* Name    : MAKE_FLAG0                                                     *
* Purpose :                                                                *
****************************************************************************
} 
Private Function MAKE_FLAG0(qr As Byte,op As Byte,aa As Byte ,tc As Byte, rd As Byte) As Byte
	result = ((qr And $01) << 7) + 
             ((op And $0F) << 3) + 
             ((aa And $01) << 2) + 
             ((tc And $01) << 1) + 
             (rd And $01)
End Function

{
****************************************************************************
* Name    : MAKE_FLAG1                                                     *
* Purpose :                                                                *
****************************************************************************
} 
Private Function MAKE_FLAG1(ra As Byte,z As Byte ,rcode As Byte) As Byte	
	result = ((ra And $01) << 7) + 
             ((z And $07) << 4) +  
             (rcode And $0F)
End Function

{
****************************************************************************
* Name    : DNSPutQuery                                                    *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub DNSPutQuery()
	PutInTXBuffer(DNS_SOCK,SentTrxID.Byt(1))   // User chosen transaction ID
	PutInTXBuffer(DNS_SOCK,SentTrxID.Byt(0))
	PutInTXBuffer(DNS_SOCK,MAKE_FLAG0(cQR_QUERY, cOP_QUERY ,0, 0, 1))  '$01)		           // Standard query with recursion
	PutInTXBuffer(DNS_SOCK,MAKE_FLAG1(0, 0, 0))                        '$00)	
	PutInTXBuffer(DNS_SOCK,$00)		           // 0x0001 questions
	PutInTXBuffer(DNS_SOCK,$01)
	PutInTXBuffer(DNS_SOCK,$00)		           // 0x0000 answers
	PutInTXBuffer(DNS_SOCK,$00)
	PutInTXBuffer(DNS_SOCK,$00)		           // 0x0000 name server resource records
	PutInTXBuffer(DNS_SOCK,$00)
	PutInTXBuffer(DNS_SOCK,$00)		           // 0x0000 additional records
	PutInTXBuffer(DNS_SOCK,$00)

	// Put hostname string to resolve
'    Select qType
'    Case cTYPE_A, cTYPE_MX

    DNSPutString()  
'    Case cTYPE_PTR
'    End Select            

	PutInTXBuffer(DNS_SOCK,(qType >> 8) And $FF ) '$00)		// Type: DNS_TYPE_A (host address) or DNS_TYPE_MX for mail exchange
	PutInTXBuffer(DNS_SOCK, qType And $FF )       'RecordType)
	PutInTXBuffer(DNS_SOCK,(qClass >> 8) And $FF )'$00)		// Class: IN (Internet)
	PutInTXBuffer(DNS_SOCK, qClass And $FF )      '$01)    
End Sub

{
****************************************************************************
* Name    : DNSResolve                                                     *
* Purpose :                                                                *
****************************************************************************
} 
Public Function DNSResolve(qStr As String, 
                           qTyp As Byte = cTYPE_A, 
                           qCls As Byte = cCLASS_IN) As DNS_INFO
    Dim len As Word
    Dim addr(4) As Byte
    Dim _port As Word
    Dim nBytes As Word
    Dim DNSHdr  As DNS_HEADER
    Dim DNSAnsw As DNS_ANSW_HDR
    Dim DummyWV As WORDVAL
    Dim DummyByt As Byte 

    If SentTrxID.Val < $FFFF Then
        Inc(SentTrxID.Val)
    Else 
        SentTrxID.Val = 0
    End If
    
    qString = qStr    
    qType = qTyp
    qClass = qCls
    
    Clear(result)

	If Not Socket(DNS_SOCK, cSn_MR_UDP, 12345, $00) Then
#if WIZ_DNS_DEBUG = true	
		USART.Write("WIZDNS: fail..",#13,#10)
#endif
        result.Status = cRC_SOCKET_ERROR
        Close(DNS_SOCK)
	Else
#if WIZ_DNS_DEBUG = true	
		USART.Write("WIZDNS: ok..",#13,#10)
#endif
        SetDestNode(DNS_SOCK, @Dns_Addr, DNS_PORT)
        
        ASyncTXBufferInit(DNS_SOCK)
    
        DNSPutQuery()
        
        ClearSUBR()

        If ASyncTXBufferSend(DNS_SOCK)= false Then
#if WIZ_DNS_DEBUG = true
            USART.Write("WIZDNS: FAIL or TIMEOUT",#13,#10) 
#endif
            result.Status = cRC_SEND_ERROR
            Close(DNS_SOCK)
            Exit       
        End If        

        WriteSocketReg(DNS_SOCK, rSn_IR, cSn_IR_SENDOK)
        SetSUBR(@Sub_Mask)
            
#if WIZ_DNS_DEBUG = true
        USART.Write("WIZDNS: wait.. ",#13,#10)
#endif
        DelayMS(1)

        SetSUBR(@Sub_Mask)

#if WIZ_DNS_DEBUG = true
	    USART.Write("WIZDNS: sent DNS QUERY",#13,#10)
#endif

        ResetDNSTimeout()
        While ReadSocketReg(DNS_SOCK, rSn_SR ) <> cSOCK_CLOSED 
            If CheckDNSTimeout() Then
#if WIZ_DNS_DEBUG = true
                USART.Write("WIZDNS: Recv timeout.",#13,#10)
#endif		    
                result.Status = cRC_TIMEOUT
                Close(DNS_SOCK)
                Exit    
            End If
            
            len = ReadSocketWord(DNS_SOCK, rSn_RX_RSR0)
    		If len  > 0 Then
#if WIZ_DNS_DEBUG = true
                USART.Write("WIZDNS: UDP len=", DecToStr(len) ,#13,#10)
#endif		    
                DNSBuff(0) = 0
                DNSBuffIdx = 0 
                nBytes = RecvFrom(DNS_SOCK,@DNSBuff,len,addr,_port)           
#if WIZ_DNS_DEBUG = true	
                USART.Write("WIZDNS: DNS addr =", AddrToString(@addr),#13,#10)
                USART.Write("WIZDNS: DNS port =", DecToStr(_port),#13,#10)
                USART.Write("WIZDNS: DNS msg nBytes=", DecToStr(nBytes) ,#13,#10)
#endif
                If ArrToLongWord(@addr)=ArrToLongWord(@Dns_Addr) And _port= DNS_PORT Then
#if WIZ_DNS_DEBUG = true	
                    USART.Write("WIZDNS: DNS TrxID =", HexToStr(DNSBuff(0),2),HexToStr(DNSBuff(1),2),#13,#10)
#endif
                    Clear(DNSHdr)
                    Clear(DNSAnsw)
                    DNSGetByte(DNSHdr.TrxID.Byt(1))
                    DNSGetByte(DNSHdr.TrxID.Byt(0))
                    If SentTrxID.Val <> DNSHdr.TrxID.Val Then
                        Close(DNS_SOCK)
                        Exit
                    End If
                    DNSGetByte(DNSHdr.Flags.Byt(1))
                    DNSGetByte(DNSHdr.Flags.Byt(0))
#if WIZ_DNS_DEBUG = true	
                    USART.Write("WIZDNS: DNS DNSHdr.flags.Val = $",HexToStr(DNSHdr.Flags.Val,4),#13,#10)
#endif
                    result.Flags  = DNSHdr.Flags.Val
                    
                    If DNSHdr.Flags.Val <> $8180 Then
                        If(DNSHdr.Flags.Byt(1) And $0F)<>0 Then
                            Select (DNSHdr.Flags.Byt(1) And $0F)
                        	Case cRC_FORMAT_ERROR
#if WIZ_DNS_DEBUG = true	
                                USART.Write("WIZDNS: DNS Error = RC_FORMAT_ERROR",#13,#10)
#endif
                                result.Status = cRC_FORMAT_ERROR
                                Close(DNS_SOCK)
                                Exit
                            Case cRC_SERVER_FAIL:
#if WIZ_DNS_DEBUG = true	
                                USART.Write("WIZDNS: DNS Error = RC_SERVER_FAIL",#13,#10)
#endif
                                result.Status = cRC_SERVER_FAIL
                                Close(DNS_SOCK)
                                Exit
                            Case cRC_NAME_ERROR
#if WIZ_DNS_DEBUG = true	
                                USART.Write("WIZDNS: DNS Error = RC_NAME_ERROR",#13,#10)
#endif
                                result.Status = cRC_NAME_ERROR
                                Close(DNS_SOCK)
                                Exit
                    		Case cRC_NOT_IMPL
#if WIZ_DNS_DEBUG = true	
                                USART.Write("WIZDNS: DNS Error = RC_NOT_IMPL",#13,#10)
#endif
                                result.Status = cRC_NOT_IMPL
                                Close(DNS_SOCK)
                                Exit
                            Case cRC_REFUSED
#if WIZ_DNS_DEBUG = true	
                                USART.Write("WIZDNS: DNS Error = RC_REFUSED",#13,#10)
#endif
                                result.Status = cRC_REFUSED
                                Close(DNS_SOCK)
                                Exit
                            Else
#if WIZ_DNS_DEBUG = true	
                                USART.Write("WIZDNS: DNS Error = UNKNOW ERROR",#13,#10)
#endif
                                result.Status = DNSHdr.Flags.Byt(1) And $0F
                                Close(DNS_SOCK)
                                Exit                            
                            End Select
                        End If    
                    Else
                        DNSGetByte(DNSHdr.Qustn.Byt(1))
                        DNSGetByte(DNSHdr.Qustn.Byt(0))
                        DNSGetByte(DNSHdr.Answr.Byt(1))
                        DNSGetByte(DNSHdr.Answr.Byt(0))
                        DNSGetByte(DNSHdr.Athrt.Byt(1))
                        DNSGetByte(DNSHdr.Athrt.Byt(0))
                        DNSGetByte(DNSHdr.Addtn.Byt(1))
                        DNSGetByte(DNSHdr.Addtn.Byt(0))

#if WIZ_DNS_DEBUG = true	
                        USART.Write("WIZDNS: DNS DNSHdr.Qustn.Val = $",HexToStr(DNSHdr.Qustn.Val,2),#13,#10)
                        USART.Write("WIZDNS: DNS DNSHdr.Answr.Val = $",HexToStr(DNSHdr.Answr.Val,2),#13,#10)
                        USART.Write("WIZDNS: DNS DNSHdr.Athrt.Val = $",HexToStr(DNSHdr.Athrt.Val,2),#13,#10)
                        USART.Write("WIZDNS: DNS DNSHdr.Addtn.Val = $",HexToStr(DNSHdr.Addtn.Val,2),#13,#10)
#endif
                        
                        While DNSHdr.Qustn.Val <> 0
                            DNSSpoolString()
                            DNSGetByte(DummyWV.Byt(1))		// Question type
                    		DNSGetByte(DummyWV.Byt(0))
                    		DNSGetByte(DummyWV.Byt(1))		// Question class
		                    DNSGetByte(DummyWV.Byt(0))
		                    DNSHdr.Qustn.Val = DNSHdr.Qustn.Val - 1
                        End While

                        // Scan through answers
                        While DNSHdr.Answr.Val <> 0
                            DNSGetByte(DNSAnsw.rName.Byt(1))		// Response name
                            DNSGetByte(DNSAnsw.rName.Byt(0))
                            DNSGetByte(DNSAnsw.rType.Byt(1))		// Response type
                            DNSGetByte(DNSAnsw.rType.Byt(0))
                            DNSGetByte(DNSAnsw.rClass.Byt(1))	// Response class
                            DNSGetByte(DNSAnsw.rClass.Byt(0))
                            DNSGetByte(DNSAnsw.rTTL.Byt(3))		// Time to live
                            DNSGetByte(DNSAnsw.rTTL.Byt(2))
                            DNSGetByte(DNSAnsw.rTTL.Byt(1))
                            DNSGetByte(DNSAnsw.rTTL.Byt(0))
                            DNSGetByte(DNSAnsw.rLen.Byt(1))		// Response length
                            DNSGetByte(DNSAnsw.rLen.Byt(0))

#if WIZ_DNS_DEBUG = true	
                            USART.Write("WIZDNS: DNS DNSAnsw.rType.Val  = $",HexToStr(DNSAnsw.rType.Val,2),#13,#10)
                            USART.Write("WIZDNS: DNS DNSAnsw.rClass.Val = $",HexToStr(DNSAnsw.rClass.Val,2),#13,#10)
                            USART.Write("WIZDNS: DNS DNSAnsw.rTTL.Val   = $",HexToStr(DNSAnsw.rTTL.Val,4),#13,#10)
                            USART.Write("WIZDNS: DNS DNSAnsw.rLen.Val   = $",HexToStr(DNSAnsw.rLen.Val,2),#13,#10)
#endif
                            // Make sure that this is a 4 byte IP address, response type A or MX, class 1
                            // Check if this is Type A or MX
                            If (DNSAnsw.rType.Val = Word(cTYPE_A) Or        
                                DNSAnsw.rType.Val = Word(cTYPE_MX)) And  '$0001 ' $000C
                                DNSAnsw.rClass.Val = Word(qClass) And    '$0001 
			                    DNSAnsw.rLen.Val = SizeOf(LongWord) Then '$0004 
			                    DNSGetByte(result.IPAddr.Byt(0))
			                    DNSGetByte(result.IPAddr.Byt(1))
                                DNSGetByte(result.IPAddr.Byt(2))
                                DNSGetByte(result.IPAddr.Byt(3))
                                result.Status = cRC_NO_ERROR 
                                'GoTo DoneSearchingRecords
#if WIZ_DNS_DEBUG = true	
                                USART.Write("WIZDNS: Resolved IP = ", AddrToString(@result.IPAddr.Byt),13,10)
#endif
                                Close(DNS_SOCK)
                                Exit
                            ElseIf DNSAnsw.rType.Val = Word(cTYPE_PTR) And 
                                   DNSAnsw.rClass.Val = Word(qClass) Then
                            Else
                                While (DNSAnsw.rLen.Val <> 0)
                                    DNSGetByte(DummyByt)
                                    DNSAnsw.rLen.Val = DNSAnsw.rLen.Val - 1
			                    End While
                            End If
                            DNSHdr.Answr.Val = DNSHdr.Answr.Val - 1
                        End While


#if WIZ_DNS_DEBUG = true	
                        USART.Write("WIZDNS: DNS Qustn = ",DecToStr(DNSHdr.Qustn.Val),#13,#10)
                        USART.Write("WIZDNS: DNS Answr = ",DecToStr(DNSHdr.Answr.Val),#13,#10)
                        USART.Write("WIZDNS: DNS Athrt = ",DecToStr(DNSHdr.Athrt.Val),#13,#10)
                        USART.Write("WIZDNS: DNS Addtn = ",DecToStr(DNSHdr.Addtn.Val),#13,#10)                       
#endif        
                    End If
                End If
            End If
        End While
#if WIZ_DNS_DEBUG = true
        USART.Write("WIZDNS: If nothing received then SOCK_CLOSED!", #13,#10)
#endif		    
        Close(DNS_SOCK)        
    End If
End Function

TimeoutCount        = cDNS_TIMEOUT
TimeoutFlag         = false
SentTrxID.Val       = $A0B1

