{
*****************************************************************************
*  Name    : UNTITLED.BAS                                                   *
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 30/10/2014                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Module W5500_DNSConsts

Public Const
    cDNS_TIMEOUT       = $FFFF 
    
{
       <Message Format>
    +---------------------+
    |        Header       |
    +---------------------+
    |       Question      | the question for the name server
    +---------------------+
    |        Answer       | Resource Records answering the question            
    +---------------------+                                        
    |      Authority      | Resource Records pointing toward an authority      
    +---------------------+                                        
    |      Additional     | Resource Records holding additional information
    +---------------------+

               As follow, example of DNS Standard Query
               +---------------------------------------------------+
    Header     | OPCODE=SQUERY                                     |
               +---------------------------------------------------+
    Question   | QNAME=SRI-NIC.ARPA., QCLASS=IN, QTYPE=A           |
               +---------------------------------------------------+
    Answer     | <empty>                                           |
               +---------------------------------------------------+
    Authority  | <empty>                                           |
               +---------------------------------------------------+
    Additional | <empty>                                           |
               +---------------------------------------------------+

               As follow, example of DNS response
               +---------------------------------------------------+
    Header     | OPCODE=SQUERY, RESPONSE, AA                       |
               +---------------------------------------------------+
    Question   | QNAME=SRI-NIC.ARPA., QCLASS=IN, QTYPE=A           |
               +---------------------------------------------------+
    Answer     | SRI-NIC.ARPA. 86400 IN A 26.0.0.73                |
               |               86400 IN A 10.0.0.51                |
               +---------------------------------------------------+
    Authority  | <empty>                                           |
               +---------------------------------------------------+
    Additional | <empty>                                           |
               +---------------------------------------------------+
}

'	QCLASS values
'
' CLASS fields appear in resource records.  The following CLASS mnemonics
' and values are defined: 

// CLASS
Public Const
    cCLASS_IN = 1, 	 // the Internet 
    cCLASS_CS = 2, 	 // the CSNET class (Obsolete)
    cCLASS_CH = 3, 	 // the CHAOS class
    cCLASS_HS = 4 	 // Hesiod [Dyer 87] 

// QCLASS
Public Const
    cQCLASS_ANY = 255 	// any class 

// QTYPE values

' TYPE fields are used in resource records.  Note that these types are a subset of QTYPEs.

// TYPE 
Public Const
    cTYPE_A     = 1,    // The ARPA Internet 
    cTYPE_NS    = 2,    // an authoritative name server 
    cTYPE_MD    = 3,    // a mail destination (Obsolete - use MX) 
    cTYPE_MF	= 4,    // a mail forwarder (Obsolete - use MX) 
    cTYPE_CNAME	= 5,    // the canonical name for an alias 
    cTYPE_SOA	= 6,    // marks the start of a zone of authority 
    cTYPE_MB	= 7,    // a mailbox domain name (EXPERIMENTAL) 
    cTYPE_MG    = 8,    // a mail group member (EXPERIMENTAL) 
    cTYPE_MR    = 9,    // a mail rename domain name (EXPERIMENTAL)
    cTYPE_NULL	= 10,   // a null RR (EXPERIMENTAL)
    cTYPE_WKS	= 11,   // a well known service description 
    cTYPE_PTR	= 12,   // a domain name pointer
    cTYPE_HINFO	= 13,   // host information 
    cTYPE_MINFO	= 14,   // mailbox or mail list information
    cTYPE_MX	= 15,   // mail exchange 
    cTYPE_TXT	= 16    // text strings 

// QTYPE 
Public Const
    cQTYPE_AXFR     = 252,  // A request for a transfer of an entire zone 
    cQTYPE_MAILB    = 253, 	// A request for mailbox-related records (MB, MG or MR) 
    cQTYPE_MAILA	= 254, 	// A request for mail agent RRs (Obsolete - see MX)
    cQTYPE_TYPE_ALL	= 255 	// A request for all records 

Public Const
    cQR_QUERY		= 0,	
    cQR_RESPONSE	= 1

Public Const
    cOP_QUERY		= 0,	// a standard query (QUERY) 
    cOP_IQUREY		= 1,	// an inverse query (IQUERY) 
    cOP_STATUS		= 2	// a server status request (STATUS)


Public Const 
    DNS_SOCK        = 2,
    DNS_PORT        = 53

Public Const
    cRC_NO_ERROR	   = 0,     // No error condition 
    cRC_FORMAT_ERROR   = 1,	    // Format error - The name server was unable to interpret the query.
    cRC_SERVER_FAIL    = 2,	    // Server failure - The name server was unable to process this 
                                //      query due to a problem with the name server.
    cRC_NAME_ERROR     = 3,     // Name Error - Meaningful only for responses from an authoritative 
                                //      name server, this code signifies that the domain name 
                                //      referenced in the query does not exist.
    cRC_NOT_IMPL       = 4,	    // Not Implemented - The name server does not support the requested 
                                //      kind of query.
    cRC_REFUSED        = 5,	    // Refused - The name server refuses to perform the specified 
                                //      operation for policy reasons. 
    cRC_SOCKET_ERROR   = 253,
    cRC_SEND_ERROR     = 254, 
    cRC_TIMEOUT        = 255        

