{
*****************************************************************************
*  Name    : W5500-Ping.BAS                                                 *
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2015 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 24/03/2015                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}


Module W5500_Ping
Include "W5500-Consts"
Include "W5500-Regs"
Include "W5500-Socket"
Include "W5500-Utils"
Include "W5500-TX"
Include "W5500-RX"
Include "convert"

#if (WIZ_PING_DEBUG = true) Or (WIZ_PING_DISPLAY = true)
    Include "Usart"
#endif

Private Const
    BUF_LEN     As Byte = 32,
    RANDOM_ID   As Word = $1234,
    RANDSEQNUM  As Word = $4321,
    CODE_ZERO   As Byte = 0

Private Dim 
    RandomID        As Word, 
    RandomSeqNum    As Word
    
Private Const
    cPING_REQUEST   = 8,
    cPING_REPLY     = 0
    
Private Dim 
    Req                     As Byte, 
    Rep                     As Byte,
    PingReplRcvd            As Boolean

{
****************************************************************************
* Name    : CheckSumCalc                                                   *
* Purpose :                                                                *
****************************************************************************
} 
Private Function CheckSumCalc(pDataBuf As Word, len As Word) As Word
    Dim sum, tsum, i, j As Word
    Dim lsum As LongWord
    j = len >> 1
    lsum = 0
    tsum = 0
    Save(FSR2)
    FSR2 = pDataBuf
    For i = 0 To (j - 1)
        tsum = POSTINC2
        tsum = tsum << 8
        tsum = tsum + POSTINC2
        lsum = lsum + tsum
    Next

    If (len Mod 2)<>0 Then
        tsum = POSTINC2
        lsum = lsum + (tsum << 8)
    End If
    
    Restore    
    sum = LongWord(lsum)
    sum = Not(sum + (lsum >> 16))
    CheckSumCalc = sum
End Function

{
****************************************************************************
* Name    : SendPingRequest                                                *
* Purpose :                                                                *
****************************************************************************
} 
Private Function SendPingRequest(s As Byte, pAddr As Word) As Boolean
    Dim i As Word
    Dim vByte(8 + BUF_LEN) As Byte
    Dim Sum As Word
    result = false
    PingReplRcvd = false
    Inc(RandomID)
    Inc(RandomSeqNum)
    vByte(0) = cPING_REQUEST
    vByte(1) = CODE_ZERO
    vByte(2) = 0
    vByte(3) = 0
    vByte(4) = RandomID.byte1
    vByte(5) = RandomID.byte0
    vByte(6) = RandomSeqNum.byte1
    vByte(7) = RandomSeqNum.byte0
    For i = 0 To (BUF_LEN - 1)
        vByte(i+8) = Byte(i Mod 8)
    Next
    Sum = 0
    Sum = CheckSumCalc(@vByte,SizeOf(vByte))
    vByte(2) = Sum.byte1 
    vByte(3) = Sum.byte0 

    SetDestNode(s, pAddr, 3000)
    ASyncTXBufferInit(s)
        
    #if WIZ_PING_DEBUG = true
    USART.Write("PING: CheckSum =", BinToStr(sum,16) ,13,10)
    #endif

    If Not PutInTXBuffer(s, @vByte, SizeOf(vByte), false) Then
        #if WIZ_PING_DEBUG = true
  		USART.Write("PING: Fail to fill TX buffer",13,10)
        #endif
        Exit
    End If
    
    If Not ASyncTXBufferSend(s) Then
        #if WIZ_PING_DEBUG = true
  		USART.Write("PING: Fail to send ping-reply packet",13,10)
        #endif
    Else
        #if WIZ_PING_DEBUG = true
        USART.Write("PING: Send Ping Request  to Destination (",
                     AddrToString(pAddr),")",13,10)
        #endif
        result = true
    End If        
End Function

{
****************************************************************************
* Name    : GetPingReply                                                   *
* Purpose :                                                                *
****************************************************************************
} 
Private Sub GetPingReply(s As Byte, len As Word)
    Dim tmp_checksum   As Word
    Dim i              As Word
	Dim rcvBuf(8 + BUF_LEN)     As Byte
	Dim rcvAddr(4)     As Byte
    Dim rcvPort        As Word  
    Dim Typ            As Byte
    Dim Code           As Byte                     
    Dim CheckSum       As Word
    Dim ID             As Word
    Dim SeqNum         As Word
    Dim Data(128)      As Byte 
    Dim Sum            As Word
    
    //* receive data from a destination */
	len = RecvFrom(s, @rcvBuf, len, rcvAddr, rcvPort)
	If len < 8 Then 
	   Exit
    End If
	
    #if WIZ_PING_DEBUG = true
    USART.Write("PING: RecvFrom=",DecToStr(len),13,10)
	#endif
		
	
	If rcvBuf(0) = cPING_REPLY Then
        #if WIZ_PING_DEBUG = true
  		USART.Write("PING: cPING_REPLY",13,10)
        #endif

        Typ 		 = rcvBuf(0)
        Code 		 = rcvBuf(1)
        CheckSum     = (rcvBuf(3)<<8) + rcvBuf(2)
        ID 		     = (rcvBuf(5)<<8) + rcvBuf(4)
        SeqNum 	     = (rcvBuf(7)<<8) + rcvBuf(6)

        i = 0
        While i < (len - 8)
            Data(i) = rcvBuf(i + 8)
            Inc(i)
        End While

        
        #if WIZ_PING_DEBUG = true
        USART.Write("PING: Recv.CheckSum =", BinToStr(CheckSum,16) ,13,10)
        #endif
        
        Sum = 0
        Sum = CheckSumCalc(@rcvBuf, len)
    
        #if WIZ_PING_DEBUG = true
        USART.Write("PING: Calc.CheckSum =", BinToStr(sum,16) ,13,10)
        #endif        


        tmp_checksum = Not Sum

		If(tmp_checksum <> $ffff) Then
            #if WIZ_PING_DEBUG = true
      		USART.Write("PING: tmp_checksum =", BinToStr(tmp_checksum,16),13,10)
            #endif
		Else
            #if WIZ_PING_DEBUG = true
      		USART.Write("PING: Reply from ",AddrToString(@rcvAddr),
      		            " ID:",DecToStr(ID), 
      		            " SeqNum:",DecToStr(SeqNum),
                        " Data size ",DecToStr(len + 6),13,10)  
            #endif
            
            #if WIZ_PING_DISPLAY = true
      		USART.Write("Ping reply from ",AddrToString(@rcvAddr),
      		            " ID:",DecToStr(ID), 
      		            " SeqNum:",DecToStr(SeqNum),
                        " Data size ",DecToStr(len + 6),13,10)              
            #endif
			PingReplRcvd = true
		End If
	ElseIf rcvBuf(0) = cPING_REQUEST Then
        #if WIZ_PING_DEBUG = true
  		USART.Write("PING: cPING_REQUEST",13,10)
        #endif
	Else     
        #if WIZ_PING_DEBUG = true
  		USART.Write("PING: Unknow Typ Message",13,10)
        #endif
    End If	
End Sub

{
****************************************************************************
* Name    : PingAuto                                                       *
* Purpose :                                                                *
****************************************************************************
} 
Public Function PingAuto(pIPAddr As Word) As Boolean
    Dim i       As Byte
    Dim cnt     As Byte
    Dim len     As Word
    Dim Sock    As Byte 
	i = 4 
    Req = 0
    Rep = 0
	cnt = 0
	len = 0 
	Sock = GetSocket(cSOCK_CLOSED, 0)
    #if WIZ_PING_DEBUG = true    				
	USART.Write("PING: Working with Socket ",DecToStr(Sock),13,10)
    #endif    					

	result = False
	While i>0 
		Select Byte(ReadSocketReg(Sock, rSn_SR))
			Case cSOCK_CLOSED
				Close(Sock)
				WriteSocketReg(Sock,rSn_PROTO,cIPPROTO_ICMP)     
				If Not Socket(Sock, cSn_MR_IPRAW, 3000, 0) Then  
                    #if WIZ_PING_DEBUG = true    				
					USART.Write("PING: Socket error",13,10)
                    #endif    					
					Exit 
				End If
		        // Check Socket register 
				While(ReadSocketReg(Sock, rSn_SR) <> cSOCK_IPRAW)
				End While
                DelayMS(2000)
			Case cSOCK_IPRAW
    			If SendPingRequest(Sock, pIPAddr) = true Then
        			Inc(Req)
        			#if WIZ_PING_DEBUG = true
        			USART.Write("PING: Req=",DecToStr(Req),13,10)
        			#endif
        			cnt = 0
        			While true
            			len = GetRecvSize(Sock) ' ReadSocketWord(0,rSn_RX_RSR0)
        				If len > 0 Then    ' len = getSn_RX_RSR(0)
            				#if WIZ_PING_DEBUG = true
        				    USART.Write("PING: Len=",DecToStr(len),13,10)
        				    #endif
        					GetPingReply(Sock, len)
        					Inc(Rep)            
        					Break
        				ElseIf(cnt > 100) Then
                            #if WIZ_PING_DEBUG = true    				
        					USART.Write("PING: Request Time out.",13,10)
                            #endif
                            #if WIZ_PING_DISPLAY = true
        					USART.Write("Ping request Time out.",13,10)
                            #endif                            					
        					cnt = 0
        					Break
        				Else
        					Inc(cnt)
        					DelayMS(50)        // wait 50ms
                        End If
            	        // wait_time for 2 seconds, Break on fail
    
    
                    End While
                End If
        End Select
		If(Req >= 3) Then
            #if WIZ_PING_DEBUG = true
	  		USART.Write("PING: Request =",DecToStr(Req),
                        " Reply =",DecToStr(Rep),13,10)
            #endif
            If(Rep = Req) Then
	  			result = true
                #if WIZ_PING_DISPLAY = true
				USART.Write("Ping completed successfully.",13,10)
                #endif
	  			Exit
	  		Else          
	  			result = false
                #if WIZ_PING_DISPLAY = true
				USART.Write("Ping with some problems.",13,10)
                #endif
	  			Exit
            End If
        End If
        Dec(i)
    End While                                       
End Function     

{
****************************************************************************
* Name    : PingCount                                                      *
* Purpose :                                                                *
****************************************************************************
}  
Public Function PingCount(SockNum As Byte, 
                          pCount As Word, 
                          pIPAddr As Word) As Boolean
    Dim rlen, cnt,i As Word
    cnt = 0
    Req = 0
    Rep = 0
	i = pCount
  	result = false
    While i<>0 	 

        #if WIZ_PING_DEBUG = true
    	If(i <> pCount) Then
	  		USART.Write("PING: Remaining ",DecToStr(i),13,10)                
        End If
        #endif

        Select (ReadSocketReg(SockNum, rSn_SR))
			Case cSOCK_CLOSED
				Close(SockNum)                 
				WriteSocketReg(SockNum,rSn_PROTO,cIPPROTO_ICMP) 
				If Socket(SockNum,cSn_MR_IPRAW,3000,0) = false Then 
                    #if WIZ_PING_DEBUG = true
            	  	USART.Write("PING: Socket Error",13,10)                
                    #endif
                    Exit
                End If
				While(ReadSocketReg(SockNum, rSn_SR) <> cSOCK_IPRAW)
				End While
                DelayMS(2000)
			Case cSOCK_IPRAW
			    If SendPingRequest(SockNum, pIPAddr) Then 
    				Inc(Req)
        			#if WIZ_PING_DEBUG = true
        			USART.Write("PING: Req=",DecToStr(Req),13,10)
        			#endif
                    cnt = 0
                    While true
                        rlen = GetRecvSize(SockNum)
        				#if WIZ_PING_DEBUG = true
    				    USART.Write("PING: rLen=",DecToStr(rlen),13,10)
    				    #endif

    					If ( rlen > 0) Then
    						GetPingReply(SockNum, rLen)
    						Inc(Rep)
    						If (PingReplRcvd) Then
                                Break
                            End If
    					End If
    
    				    // wait_time For 2 seconds, Break on fail
    					If ( (cnt > 100) ) Then
    	                    #if WIZ_PING_DEBUG = true
                    	  	USART.Write("PING: Request Timeout.",13,10)                
                            #endif
                            #if WIZ_PING_DISPLAY = true
    				        USART.Write("Ping request timeout.",13,10)
                            #endif
    						cnt = 0
    						Break
    					Else  
    						Inc(cnt)
    						DelayMS(50)  
    					End If
    				End While
                End If		
        End Select
        #if WIZ_PING_DEBUG = true
  		USART.Write("PING: Requests =",DecToStr(Req),
                    " Replies =",DecToStr(Rep),13,10)
        #endif
        #if WIZ_PING_DISPLAY = true
        USART.Write("Ping requests ",DecToStr(Req),
                    " and replies ",DecToStr(Rep),13,10)
        #endif
        DelayMS(1000)
        Dec(i)
    End While
    If(Req=Rep) Then
		result = true
  		#if WIZ_PING_DEBUG = true
  		USART.Write("PING: Ping requests completed successfully",13,10)
        #endif
        #if WIZ_PING_DISPLAY = true
  		USART.Write("Ping requests completed successfully.",13,10)
        #endif
	Else          
		result = false
  		#if WIZ_PING_DEBUG = true
  		USART.Write("PING: Some Ping requests with problems.",13,10)        
        #endif
        #if WIZ_PING_DISPLAY = true
  		USART.Write("Some Ping requests with problems.",13,10)
        #endif
    End If
End Function

Req             = 0
Rep             = 0
PingReplRcvd    = false
RandomID        = $1234
RandomSeqNum    = $4321


