{
*****************************************************************************
*  Name    : W5500-RX.BAS                                                   * 
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 16/10/2014                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Module W5500_RX

Include "W5500-Consts"
Include "W5500-Regs"
Include "W5500-utils"
Include "utils"
Include "string"
Include "usart"
Include "convert"

Private Dim
    WriteSocketReg          As W5500_Regs.WriteSocketReg,
    WriteCommonReg          As W5500_Regs.WriteCommonReg,
    WritesocketWord         As W5500_Regs.WritesocketWord,
    WriteCommonWord         As W5500_Regs.WriteCommonWord,
    ReadSocketReg           As W5500_Regs.ReadSocketReg,
    ReadCommonReg           As W5500_Regs.ReadCommonReg,    
    ReadSocketword          As W5500_Regs.ReadSocketword,
    GetBase                 As W5500_Consts.GetBase

{
****************************************************************************
* Name    : GetRecvSize                                                    *
****************************************************************************
} 
Public Function GetRecvSize(sock As Byte) As Word
    result = ReadSocketword(sock,rSn_RX_RSR0)
End Function

{
****************************************************************************
* Name    : GetRXBufferSize                                                *
****************************************************************************
} 
Public Function GetRXBufferSize(sock As Byte) As Word
    If sock > ( MAX_SOCK_NUM - 1 ) Then
        result = 0
        Exit
    End If
    result = (1024 * ReadSocketReg(sock , rSn_RXBUF_SIZE))
End Function


{
****************************************************************************
* Name    : ReleaseRXBuffer                                                *
****************************************************************************
} 
Public Sub ReleaseRXBuffer(sock As Byte)
    WritesocketWord(sock, rSn_RX_RD0, Word(ReadSocketword( sock, rSn_RX_RD0 ) + GetRecvSize(sock))) 			    
    WriteSocketReg(sock, rSn_CR, cSn_CR_RECV)    
    While(ReadSocketReg(sock, rSn_CR) = 1 )
    End While                             
End Sub

{
****************************************************************************
* Name    : GetRXBuffer                                                    *
****************************************************************************
} 
Public Sub GetRXBuffer(sock As Byte, 
                       pRxBuffAddr As Word, 
                       buflen As Word, 
                       Offset As Word = 0)
    Dim src As Word
    
    src = ReadSocketword( sock, rSn_RX_RD0 )
    ReadSocketBuf(sock, src, pRxBuffAddr + Offset, buflen) 
    Inc(src, buflen)    
    WritesocketWord(sock, rSn_RX_RD0, src) 			    
    WriteSocketReg(sock, rSn_CR, cSn_CR_RECV)
    While(ReadSocketReg(sock, rSn_CR) = 1 )
    End While
End Sub

Public Function GetRXByte(sock As Byte) As Byte 
    Dim src As Word
    src = ReadSocketword(sock, rSn_RX_RD0) 
    ReadSocketBuf(sock, src, @result, 1) 
    Inc(src)     
    WritesocketWord(sock, rSn_RX_RD0, src) 			    
    WriteSocketReg(sock, rSn_CR, cSn_CR_RECV)
    While(ReadSocketReg(sock, rSn_CR) = 1 )
    End While                             
End Function

Public Function SeekRXByte(sock As Byte, Offset As Word = 0) As Byte 
    Dim src As Word
    src = ReadSocketword(sock, rSn_RX_RD0) + OffSet 
    ReadSocketBuf(sock, src, @result, 1) 
End Function


Public Sub MoveRXBufferPtr(sock As Byte,
                           IncPos As Word)
    Dim src As Word
    src = ReadSocketword( sock, rSn_RX_RD0 )
    Inc(src, IncPos)    
    WritesocketWord(sock, rSn_RX_RD0, src) 			    
    WriteSocketReg(sock, rSn_CR, cSn_CR_RECV)
    While(ReadSocketReg(sock, rSn_CR) = 1 )
    End While                             
End Sub   


{
****************************************************************************
* Name    : ExtractRXBuffer                                                *
****************************************************************************
} 
Public Sub ExtractRXBuffer(sock As Byte, 
                           pBuffAddr As Word, 
                           pStart As Integer, 
                           buflen As Word, 
                           EOS As Boolean = false ) 
    ReadSocketBuf(sock, Word(pStart), pBuffAddr , buflen, EOS) 
End Sub

{
****************************************************************************
* Name    : RecvFrom                                                       *
* Purpose :                                                                *
****************************************************************************
} 
Public Function RecvFrom(sock As Byte, 
                         buf As Word, 
                         len As Word, 
                         ByRef addr() As Byte, 
                         ByRef _port As Word ) As Word
    Dim head(8) As Byte
    Dim data_len As Word
    Dim ptr As Word
    data_len = 0
    If ( len > 0 ) Then
        ptr = ReadSocketword(sock,rSn_RX_RD0)
        
        Select (ReadSocketReg(sock, rSn_MR) And $07)
        Case cSn_MR_UDP 
            ExtractRXBuffer(sock, @head, Integer(ptr), Integer(8)) 
            Inc(ptr, 8)
            ArrCpy(@addr, @head, 4) 
            _Port = head(4)
            _Port = (_Port << 8) + head(5)
            data_len = head(6)
            data_len = (data_len << 8) + head(7)
            ExtractRXBuffer(sock, buf, Integer(ptr), Integer(data_len)) 
            Inc(ptr, data_len)
            WritesocketWord(sock,rSn_RX_RD0, ptr) 
        Case cSn_MR_MACRAW 
            ExtractRXBuffer(sock, @head, Integer(ptr), Integer(2)) 
            Inc( ptr, 2)
            data_len = head(0)
            data_len = (data_len<<8) + head(1) - 2
            ExtractRXBuffer(sock, buf, Integer(ptr), Integer(data_len)) 
            Inc( ptr, data_len)
            WritesocketWord(sock,rSn_RX_RD0, ptr)
        Case cSn_MR_IPRAW
            ExtractRXBuffer(sock, @head, Integer(ptr), Integer(6)) 
            Inc(ptr, 6)
            ArrCpy(@addr, @head, 4) 
            data_len = head(4)
            data_len = (data_len << 8) + head(5)
            ExtractRXBuffer(sock, buf, Integer(ptr), Integer(data_len)) 
            Inc(ptr, data_len)
            WritesocketWord(sock,rSn_RX_RD0, ptr)                          
        Else
        End Select
        WriteSocketReg(sock, rSn_CR, cSn_CR_RECV)
        While(ReadSocketReg(sock, rSn_CR) = 1 )
        End While
    End If
    result = data_len
End Function

Public Function SeekToken(SockNum As Byte,
                          OffsP As Word,
                          pSubStr As String) As Integer
    Dim SubIndex, 
        SubLength As Byte
    Dim Src As Word,
        LastP As Word
        
    result = -1
    SubIndex = 0
    SubLength = Length(pSubStr)
    Src = Word(ReadSocketword(SockNum, rSn_RX_RD0)) + OffsP    
    LastP = Word(ReadSocketword(SockNum, rSn_RX_RSR0)) - OffsP 
	While lastP > 0
	    If Char(SeekSocketBufByte(SockNum, src)) = pSubStr(SubIndex) Then 
            Inc(SubIndex)
            If SubIndex = SubLength Then
                result = Integer(src - SubLength + 1)
            End If
        Else
            SubIndex = 0
        End If
        Dec(LastP) 
        Inc(src)
	End While
 End Function

