{
*****************************************************************************
*  Name    : W5500-Regs.BAS                                                 *
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 16/10/2014                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}

Module W5500_Regs

Include "SPI"
Include "W5500-Config"
Include "W5500-Consts"
Include "convert"
Include "usart"

Private Dim
    SCS_PIN As WIZ_SCS_PIN.WIZ_SCS_PIN@,
    RST_PIN As WIZ_RST_PIN.WIZ_RST_PIN@


{
****************************************************************************
* Name    : WizWriteBuf                                                    *
****************************************************************************
} 
Private Sub WizWriteBuf(Block As Byte, WizAddr As Word, pBufAddr As Word, len As Word)
    Dim BlkOP As Byte
    BlkOP = (Block <<3 ) Or %00000100   ' Scrittura & VDM
    Low(SCS_PIN) 
    SPI.WriteByte(WizAddr.byte1)
    SPI.WriteByte(WizAddr.byte0)
    SPI.WriteByte(BlkOP)    
    Save(FSR2)
    FSR2 = pBufAddr
    While len > 0 
        SPI.WriteByte(Byte(POSTINC2))
        len = len - 1
    End While
    Restore
    High(SCS_PIN)
End Sub

{
****************************************************************************
* Name    : WizReadBuf                                                     *
****************************************************************************
} 
Private Sub WizReadBuf(Block As Byte, 
                       WizAddr As Word, 
                       pBufAddr As Word, 
                       len As Word, 
                       EOS As Boolean = false)
    Dim BlkOP As Byte
    BlkOP = (Block <<3 )  
    Low(SCS_PIN)
    SPI.WriteByte(WizAddr.byte1)
    SPI.WriteByte(WizAddr.byte0)
    SPI.WriteByte(BlkOP)    
    Save(FSR2)
    FSR2 = pBufAddr
    While len > 0 
        POSTINC2 = Byte(SPI.ReadByte())
        len = len - 1
    End While
    If EOS Then
        POSTINC2 = 0    
    End If
    Restore
    High(SCS_PIN)
End Sub


{
****************************************************************************
* Name    : WizWrite                                                       *
****************************************************************************
} 
Private Sub WizWrite(Block As Byte, WizAddr As Word, pData As Byte)
    Dim BlkOP As Byte
    BlkOP = (Block <<3 ) Or %00000100   ' Scrittura & VDM
    Low(SCS_PIN) 
    SPI.WriteByte(WizAddr.byte1)
    SPI.WriteByte(WizAddr.byte0)
    SPI.WriteByte(BlkOP)    
    SPI.WriteByte(pData)
    High(SCS_PIN)
End Sub

{
****************************************************************************
* Name    : WizRead                                                        *
****************************************************************************
} 
Private Function WizRead(Block As Byte, ByVal WizAddr As Word) As Byte
    Dim BlkOP As Byte
    BlkOP = (Block <<3 ) 
    Low(SCS_PIN)
    SPI.WriteByte(WizAddr.byte1)
    SPI.WriteByte(WizAddr.byte0)
    SPI.WriteByte(BlkOP)    
    Result = SPI.ReadByte()
    High(SCS_PIN)
End Function

{
****************************************************************************
* Name    : WriteSocketReg                                                 *
****************************************************************************
} 
Public Sub WriteSocketReg(Sock As Byte, Reg As Byte , pData As Byte)
    WizWrite(GetBase(sock), Word(Reg), pData)
End Sub

{
****************************************************************************
* Name    : WriteCommonReg                                                 *
****************************************************************************
} 
Public Sub WriteCommonReg(Reg As Byte , pData As Byte)
    WizWrite(Byte(0), Word(Reg), pData)
End Sub

{
****************************************************************************
* Name    : WriteSocketWord                                                *
****************************************************************************
} 
Public Sub WriteSocketWord(sock As Byte, reg As Byte ,pData As Word)
    WriteSocketReg(sock, reg,(pData And $FF00) >> 8 )
    WriteSocketReg(sock, reg + 1,(pData And $00FF))
End Sub

{
****************************************************************************
* Name    : WriteCommonWord                                                *
****************************************************************************
} 
Public Sub WriteCommonWord(sock As Byte, reg As Byte ,pData As Word)
    WriteCommonReg(reg,(pData And $FF00) >> 8 )
    WriteCommonReg(reg + 1,(pData And $00FF))
End Sub

{
****************************************************************************
* Name    : WriteSocketPtr                                                 *
****************************************************************************
} 
Public Sub WriteSocketPtr(Sock As Byte, Reg As Byte , pBufAddr As Word, Len As Word)
    WizWriteBuf(GetBase(sock), Word(Reg), pBufAddr, Len)
End Sub

{
****************************************************************************
* Name    : WriteCommonPtr                                                 *
****************************************************************************
} 
Public Sub WriteCommonPtr(Reg As Byte , pBufAddr As Word, Len As Word)
    WizWriteBuf(Byte(0), Word(Reg), pBufAddr, Len)
End Sub

{
****************************************************************************
* Name    : WriteSocketBuf                                                 *
****************************************************************************
} 
Public Sub WriteSocketBuf(Sock As Byte, Addr As Word , pBufAddr As Word, Len As Word)
    WizWriteBuf(TXBUF_BLOCK(sock), Addr, pBufAddr, Len)
End Sub
                           
{
****************************************************************************
* Name    : ReadSocketReg                                                  *
****************************************************************************
} 
Public Function ReadSocketReg(Sock As Byte, Reg As Byte ) As Byte
    result = WizRead(GetBase(sock), Word(Reg))
End Function

{
****************************************************************************
* Name    : ReadCommonReg                                                  *
****************************************************************************
} 
Public Function ReadCommonReg(Reg As Byte) As Byte
    result = WizRead(Byte(0), Word(Reg))
End Function

{
****************************************************************************
* Name    : ReadSocketWord                                                 *
****************************************************************************
} 
Public Function ReadSocketWord(sock As Byte, reg As Byte) As Word
    Result = ReadSocketReg(sock, reg) 
    Result = ((result And $00FF) << 8)  + ReadSocketReg(sock, reg + 1)
End Function

{
****************************************************************************
* Name    : ReadCommonPtr                                                  *
****************************************************************************
} 
Public Sub ReadCommonPtr(Reg As Byte , pBufAddr As Word, Len As Word)
    WizReadBuf(Byte(0), Word(Reg), pBufAddr, Len)
End Sub

{
****************************************************************************
* Name    : ReadSocketBuf                                                  *
****************************************************************************
} 
Public Sub ReadSocketBuf(Sock As Byte, 
                         Addr As Word , 
                         pBufAddr As Word, 
                         Len As Word, 
                         EOS As Boolean = false)
    If len = 0 Then                         
        len = ReadSocketWord(sock,rSn_RX_RSR0) - addr                         
    End If
    WizReadBuf(RXBUF_BLOCK(sock), Addr, pBufAddr, Len, EOS)
End Sub

Public Function SeekSocketBufByte(Sock As Byte, Addr As Word) As Byte
    result = WizRead(RXBUF_BLOCK(sock), Addr)
End Function


{
****************************************************************************
* Name    : SetDestNode                                                    *
****************************************************************************
} 
Public Sub SetDestNode(sock As Byte, pArrIPAddr As Word, dport As Word)
	WriteSocketPtr(sock, rSn_DIPR0, pArrIPAddr, 4)
	WriteSocketReg(sock, rSn_DPORT0, Byte(dport>>8))
	WriteSocketReg(sock, rSn_DPORT0 + 1, Byte(dport And $00FF))
End Sub

{
****************************************************************************
* Name    : SetGAR                                                         *
****************************************************************************
} 
Public Sub SetGAR(pAddr As Word)
	WriteCommonPtr(rGAR0, pAddr, 4)
End Sub

{
****************************************************************************
* Name    : SetSUBR                                                        *
****************************************************************************
} 
Public Sub SetSUBR(pAddr As Word)
	WriteCommonPtr(rSUBR0, pAddr, 4)
End Sub


{
****************************************************************************
* Name    : SetSIPR                                                        *
****************************************************************************
} 
Public Sub SetSIPR(pAddr As Word)
	WriteCommonPtr(rSIPR0, pAddr, 4)
End Sub


{
****************************************************************************
* Name    : SetSHAR                                                        *
****************************************************************************
} 
Public Sub SetSAR(ByRefConst MacAddr() As Byte)
	WriteCommonReg(rSHAR0, MacAddr(0))
	WriteCommonReg(rSHAR0 + 1, MacAddr(1))
	WriteCommonReg(rSHAR0 + 2, MacAddr(2))
	WriteCommonReg(rSHAR0 + 3, MacAddr(3))
	WriteCommonReg(rSHAR0 + 4, MacAddr(4))
	WriteCommonReg(rSHAR0 + 5, MacAddr(5))
End Sub

Private Dim Dummy(6) As Byte

{
****************************************************************************
* Name    : SIPRtoString                                                   *
****************************************************************************
} 
Public Function SIPRtoString() As String 
    ReadCommonPtr(rSIPR0, @Dummy, 4)                   
    result = DecToStr(Dummy(0)) + "." +
             DecToStr(Dummy(1)) + "." +
             DecToStr(Dummy(2)) + "." +
             DecToStr(Dummy(3)) 
End Function

{
****************************************************************************
* Name    : SUBRtoString                                                   *
****************************************************************************
} 
Public Function SUBRtoString() As String 
    ReadCommonPtr(rSUBR0, @Dummy, 4)                   
    result = DecToStr(Dummy(0)) + "." +
             DecToStr(Dummy(1)) + "." +
             DecToStr(Dummy(2)) + "." +
             DecToStr(Dummy(3)) 
End Function

{
****************************************************************************
* Name    : GARtoString                                                    *
****************************************************************************
} 
Public Function GARtoString() As String                     
    ReadCommonPtr(rGAR0, @Dummy, 4)                   
    result = DecToStr(Dummy(0)) + "." +
             DecToStr(Dummy(1)) + "." +
             DecToStr(Dummy(2)) + "." +
             DecToStr(Dummy(3)) 
End Function

{
****************************************************************************
* Name    : SARtoString                                                    *
****************************************************************************
} 
Public Function SARtoString() As String 
    ReadCommonPtr(rSHAR0, @Dummy, 6)                   
    result = HexToStr(Dummy(0),2) + ":" +
             HexToStr(Dummy(1),2) + ":" +
             HexToStr(Dummy(2),2) + ":" +
             HexToStr(Dummy(3),2) + ":" +
             HexToStr(Dummy(4),2) + ":" +
             HexToStr(Dummy(5),2)
End Function

{
****************************************************************************
* Name    : SysMemInit                                                     *
****************************************************************************
} 
Public Function SysMemInit() As Boolean
    Dim i As Byte
    Dim tmp As Byte
    result = false
    tmp = 0
    For i = 0  To MAX_SOCK_NUM -1 
        Inc(tmp, SOCK_TX_SIZES(i))
    Next 
    If tmp > 16 Then 
        Exit 
    End If
    For i = 0  To MAX_SOCK_NUM -1 
        WriteSocketReg(i, rSn_TXBUF_SIZE, SOCK_TX_SIZES(i)) 
    Next 
    tmp = 0
    For i = 0  To MAX_SOCK_NUM -1 
        Inc(tmp, SOCK_RX_SIZES(i))
    Next 
    If tmp > 16 Then 
        Exit 
    End If
    For i = 0  To MAX_SOCK_NUM -1 
        WriteSocketReg(i, rSn_RXBUF_SIZE, SOCK_RX_SIZES(i)) 
    Next 
    result = true
End Function

{
****************************************************************************
* Name    : InitW5500                                                      *
****************************************************************************
} 
Private Sub InitW5500()
    WriteCommonReg(rMR,$80)
    SetGAR(@Gtw_Addr)   // Assegno lindirizzo del Gateway   
    SetSAR(cMac_Addr)   // Assegno il MAC address
    SetSUBR(@Sub_Mask)  // Imposto il Subnet Mask Address 
    SetSIPR(@Ip_Addr)   // Assegno lindirizzo IP    
    SysMemInit()
    DelayMS(1000)
End Sub

{
****************************************************************************
* Name    : Initialize                                                     *
****************************************************************************
} 
Private Sub Initialize()
    Output(SCS_PIN)     
    Output(RST_PIN)
    Low(RST_PIN) 
    High(SCS_PIN)
    DelayUS(500)                    // 20/3/2015
    High(RST_PIN)
    DelayUS(500)                    // 20/3/2015
    SPI.SetAsMaster(spiOscDiv4)        
    SPI.SetClock(spiIdleLow, spiRisingEdge)
    SPI.SetSample(spiSampleEnd)
    SPI.EnableSPI() 
    DelayUS(500)
    High(RST_PIN)          
    DelayMS(1000)
End Sub

Initialize()
InitW5500()


