{
*****************************************************************************
*  Name    : W5500-Socket.BAS                                               *
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 16/10/2014                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}

Module W5500_Socket
Include "W5500-Consts"
Include "W5500-Regs"
Include "usart"
Include "convert"
Include "EEPROM.bas"

#option TCP_PORT_ROM_ADDR = 0

Private Const 
    iniTcpPort   As Word = 1000,
    endTcpPort   As Word = 2000,
    eePortAddr   = TCP_PORT_ROM_ADDR
{
****************************************************************************
* Name    : Close                                                          *
****************************************************************************
} 
Public Sub Close(sock As Byte)
    WriteSocketReg(sock, rSn_CR, cSn_CR_CLOSE)
    While(ReadSocketReg(sock, rSn_CR) = 1)
    End While
End Sub

{
****************************************************************************
* Name    : Disconnect                                                     *
****************************************************************************
} 
Public Sub Disconnect(sock As Byte)
    WriteSocketReg(sock, rSn_CR, cSn_CR_DISCON)
    While(ReadSocketReg(sock, rSn_CR) = 1 )
    End While
End Sub


{
****************************************************************************
* Name    : GetSocket                                                      *
* Purpose :                                                                *
****************************************************************************
} 
Public Function GetSocket(status As Byte, start As Byte) As Byte
    Dim i As Byte
	If start > (MAX_SOCK_NUM - 1) Then
        start = 0
    End If

	For i = start To MAX_SOCK_NUM 
        If ReadSocketReg(i,rSn_SR) = status Then
            result = i
            Exit 
        End If
    Next
    result = MAX_SOCK_NUM
End Function


{
****************************************************************************
* Name    : Socket                                                         *
* Purpose :                                                                *
****************************************************************************
} 
Public Function Socket(sock As Byte, protocol As Byte, _port As Word, flag As Byte) As Boolean
    result = false
    If ((protocol = cSn_MR_TCP) Or 
        (protocol = cSn_MR_UDP) Or 
        (protocol = cSn_MR_MACRAW) Or 
        (protocol = cSn_MR_IPRAW)) Then
        
        Close(sock)
        WriteSocketReg(sock, rSn_MR,Byte( protocol Or ( flag And $F0)))
        If _port <> 0 Then
            WriteSocketWord(sock, rSn_PORT0, _port) 
        Else
            WriteSocketWord(sock, rSn_PORT0, TCP_PORT)
        End If
        
        WriteSocketReg(sock, rSn_CR, cSn_CR_OPEN)

        While(ReadSocketReg(sock,rSn_CR) = 1 )    // 30/11/2014
        End While
         
        If protocol = cSn_MR_UDP Then
            result = true
            Exit
        ElseIf protocol = cSn_MR_IPRAW Then
            If ReadSocketReg(sock, rSn_SR) = cSOCK_IPRAW Then
                result = true
            End If
            Exit
        Else            
            If ReadSocketReg(sock, rSn_SR) = cSOCK_INIT Then
                Result = true
                Exit
            Else
                Close(sock)   
                Result = false
                Exit
            End If            
        End If
    End If
End Function

{
****************************************************************************
* Name    : ComputeLocalPort                                               *
* Purpose :                                                                *
****************************************************************************
} 
Private Function ComputeLocalPort() As Word      // 20/3/2015
    result = EE.ReadWord(eePortAddr)
    If (result < iniTcpPort) Or (result > endTcpPort) Then
        result = iniTcpPort
    Else
        Inc(result) 
    End If
    EE.WriteWord(eePortAddr, result)
End Function

{
****************************************************************************
* Name    : Connect                                                        *
* Purpose :                                                                *
****************************************************************************
} 
Public Function Connect(sock As Byte, 
                        pArrIPAddr As Word, 
                        _port As Word, 
                        flags As Byte) As Boolean
    result = false
    If Socket(sock, cSn_MR_TCP, ComputeLocalPort(), flags) Then  
        SetDestNode(sock, pArrIPAddr, _port)
        
        WriteSocketReg(sock, rSn_CR, cSn_CR_CONNECT)    

        While(ReadSocketReg(sock,rSn_CR) = 1 )    // 30/11/2014
        End While

        // BLOCCO IN SOCK_SYNSENT
            
        While ReadSocketReg(sock, rSn_SR)<> cSOCK_ESTABLISHED  'And 
             ' ReadSocketReg(sock, rSn_SR)<> cSOCK_SYNSENT
            If (ReadSocketReg(sock, rSn_IR) And cSn_IR_TIMEOUT ) <> 0 Then
                result = false 
                Exit
            End If
        End While     
        result = true

    End If
End Function


