{
*****************************************************************************
*  Name    : W5500-TX.BAS                                                   *
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 16/10/2014                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Module W5500_TX

Include "W5500-Consts"
Include "W5500-Regs"
Include "W5500-Socket"
Include "W5500-Utils"
Include "convert"
Include "string"
Include "usart"

Private Dim
    WriteSocketReg          As W5500_Regs.WriteSocketReg,
    WriteCommonReg          As W5500_Regs.WriteCommonReg,
    WriteSocketWord         As W5500_Regs.WriteSocketWord,
    WriteCommonWord         As W5500_Regs.WriteCommonWord,
    ReadSocketReg           As W5500_Regs.ReadSocketReg,
    ReadCommonReg           As W5500_Regs.ReadCommonReg,    
    ReadSocketword          As W5500_Regs.ReadSocketword,
'    ReadCommonword          As W5500_Regs.ReadCommonword,    
    GetBase                 As W5500_Consts.GetBase, 
    Disconnect              As W5500_Socket.Disconnect

Private Dim bufPos(MAX_SOCK_NUM) As Word

{
****************************************************************************
* Name    : GetTXBufferSize                                                *
****************************************************************************
} 
Public Function GetTXBufferSize(sock As Byte) As Word
    If sock > ( MAX_SOCK_NUM - 1 ) Then
        result = 0
        Exit
    End If
    result = (1024 * ReadSocketReg(sock , rSn_TXBUF_SIZE))
End Function


{
****************************************************************************
* Name    : CloseTXBuffer                                                  *
****************************************************************************
} 
Private Sub CloseTXBuffer(sock As Byte,ptr As Word)
    WriteSocketWord( sock, rSn_TX_WR0 , ptr )
End Sub

{
****************************************************************************
* Name    : SendTXBuffer                                                   *
****************************************************************************
} 
Private Function SendTXBuffer(sock As Byte) As Boolean
    result = false
    WriteSocketReg(sock, rSn_CR, cSn_CR_SEND)
    While(ReadSocketReg(sock, rSn_CR) = 1 )
    End While  

      
    While ( (ReadSocketReg(sock,rSn_IR) And cSn_IR_SENDOK) <> cSn_IR_SENDOK )
        If (ReadSocketReg(sock,rSn_IR) And cSn_IR_TIMEOUT) <> 0 Then
            WriteSocketReg(sock, rSn_IR, (cSn_IR_SENDOK Or cSn_IR_TIMEOUT)) 
            Exit 
        End If
    End While  
    result = true
End Function

{
****************************************************************************
* Name    : ASyncTXBufferInit                                              *
****************************************************************************
} 
Public Sub ASyncTXBufferInit(sock As Byte)
    bufPos(sock) = ReadSocketword(sock, rSn_TX_WR0)    
End Sub

{
****************************************************************************
* Name    : ASyncTXBufferSend                                              *
****************************************************************************
} 
Public Function ASyncTXBufferSend(sock As Byte) As Boolean
    CloseTXBuffer(sock,bufPos(sock))
    result = SendTXBuffer(sock)   
End Function

{
****************************************************************************
* Name    : PutInTXBuffer                                                  *
****************************************************************************
} 
Public Function PutInTXBuffer(sock As Byte, 
                              pAddr As Word , 
                              buflen As Word, 
                              SendAndClose As Boolean = true) As Boolean
    Dim ptr As Word
    
    result = false
    
    If (buflen <= 0) Then 
        Exit 
    End If

    If SendAndClose = false Then
        ptr = bufPos(sock)
    Else
        ptr = ReadSocketword(sock, rSn_TX_WR0)
    End If
    
    WriteSocketBuf(sock, ptr, pAddr, buflen )
    Inc(ptr, buflen)
    
    
    If SendAndClose Then
        CloseTXBuffer(sock,ptr)
        result = SendTXBuffer(sock)   
    Else
        bufPos(sock) = ptr
        result = true
    End If
End Function

Public Function PutInTXBuffer(sock As Byte, 
                              pText As String , 
                              SendAndClose As Boolean = true) As Boolean
    result = PutInTXBuffer(sock, @pText, Word(Length(pText)), SendAndClose)
End Function

Public Function PutInTXBuffer(sock As Byte, 
                              bValue As Byte) As Boolean
    result = PutInTXBuffer(sock,@bValue,1,false)                                
End Function

{
****************************************************************************
* Name    : ClearSUBR                                                      *
* Purpose :                                                                *
****************************************************************************
} 
Public Sub ClearSUBR()
    Dim Ip_Var(4) As Byte
    
	Ip_Var(0) = ReadCommonReg(rSIPR0)
	Ip_Var(1) = ReadCommonReg(rSIPR0 + 1)
	Ip_Var(2) = ReadCommonReg(rSIPR0 + 2)
	Ip_Var(3) = ReadCommonReg(rSIPR0 + 3)

	
	If((Ip_Var(0)=0 And Ip_Var(1)=0 And Ip_Var(2)=0 And Ip_Var(3)=0)) Then
		WriteCommonReg(rSUBR0 + 0, 0)
		WriteCommonReg(rSUBR0 + 1, 0)
		WriteCommonReg(rSUBR0 + 2, 0)
		WriteCommonReg(rSUBR0 + 3, 0)
    End If    
End Sub

{
****************************************************************************
* Name    : SendTo                                                         *
* Purpose :                                                                *
****************************************************************************
} 
Public Function SendTo(sock As Byte, 
                       buf As Word, 
                       len As Word, 
                       ByRef addr() As Byte , 
                       _port As Word ) As Word
    Dim isr As Byte
    Dim ret As Word
    isr = 0
    ret = 0
                  
    If (len > ( ReadSocketReg(sock,rSn_TXBUF_SIZE) << 10 ) ) Then 
        ret = ( ReadSocketReg(sock,rSn_TXBUF_SIZE) << 10 )
    Else 
        ret = len
    End If 
    If ( addr(0) = 0 And 
         addr(1) = 0 And 
         addr(2) = 0 And 
         addr(3) = 0 ) Or (_port = 0) Or (ret = 0) Then
        ret = 0
    Else
        SetDestNode(sock, @addr, _port)
        PutInTXBuffer(sock, buf, len, false)
        WriteSocketReg(sock, rSn_CR, cSn_CR_SEND)
        While(ReadSocketReg(sock, rSn_CR) = 1 )
        End While    

        While ( (ReadSocketReg(sock,rSn_IR) And cSn_IR_SENDOK) <> cSn_IR_SENDOK )
            If (ReadSocketReg(sock,rSn_IR) And cSn_IR_TIMEOUT) <> 0 Then
                WriteSocketReg(sock, rSn_IR, (cSn_IR_SENDOK Or cSn_IR_TIMEOUT))  
                result = 0
                Exit 
            End If
        End While
        WriteSocketReg(sock, rSn_IR, cSn_IR_SENDOK)
    End If
	result = ret
End Function


Dim i As Byte
For i = 0 To MAX_SOCK_NUM -1
  bufPos(i) =  0
Next


