{
*****************************************************************************
*  Name    : W5500.BAS                                                      *
*  Author  : M.Cerutti                                                      *
*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS]              *
*          : All Rights Reserved                                            *
*  Date    : 16/03/2015                                                     *
*  Version : 2.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Module W5500

Include "W5500-Consts"
Include "W5500-TX"
Include "W5500-RX"
Include "W5500-Regs"
Include "W5500-Socket"
Include "W5500-Utils"
Include "string"
Include "usart"
Include "convert"

// validate W5500 ports ...
#if IsOption(WIZ_SCS_PIN) 
   #if Not IsValidPortPin(WIZ_SCS_PIN)   
      #error WIZ_SCS_PIN, "Invalid port pin for WIZ_SCS_PIN."
   #endif
#else
   #error "Please set WIZ_SCS_PIN on valid port pin."    
#endif

#if IsOption(WIZ_RST_PIN) 
   #if Not IsValidPortPin(WIZ_RST_PIN)   
      #error WIZ_RST_PIN, "Invalid port pin for WIZ_RST_PIN."
   #endif
#else
   #error "Please set WIZ_RST_PIN on valid port pin."    
#endif

#if _mssp = 0 
   #error _device,"This device does not have have an hardware SPI port."        
#endif

#if IsOption(SOCK_TIMEOUT) 
    Const cSOCK_TIMEOUT As Word = SOCK_TIMEOUT
#else
    Const cSOCK_TIMEOUT As Word = 40000
#endif

#option USE_DHCP = true
#if USE_DHCP = true
    Include "W5500-DHCP"
#endif

#option USE_DNS = false
#IF USE_DNS = true
    Include "W5500-DNS"
#endif

#option USE_PING = True
#IF USE_PING = true
    Include "W5500-Ping"
#endif


Public Dim 

#if USE_DHCP = true
    DHCPSetEvents           As W5500_DHCP.SetEvents,
    DHCPTask                As W5500_DHCP.DHCPTask,
#endif

#IF USE_DNS = true
    DNS_INFO                As W5500_DNS.DNS_INFO,
    DNSResolve              As W5500_DNS.DNSResolve,
#endif

#if USE_PING = true
    PingAuto                As W5500_Ping.PingAuto,
    PingCount               As W5500_Ping.PingCount,
'    CheckSumCalc            As W5500_Ping.CheckSumCalc,
'    GetCheckSum             As W5500_Ping.GetCheckSum,
'    CheckSumInit            As W5500_Ping.CheckSumInit,    
#endif
    PutInTXBuffer           As W5500_TX.PutInTXBuffer,
    ASyncTXBufferInit       As W5500_TX.ASyncTXBufferInit,
    ASyncTXBufferSend       As W5500_TX.ASyncTXBufferSend,
    GetTXBufferSize         As W5500_TX.GetTXBufferSize,
    DataAvailable           As W5500_RX.GetRecvSize,
    GetRXBuffer             As W5500_RX.GetRXBuffer,
    GetRXBufferSize         As W5500_RX.GetRXBufferSize,
    ReleaseRXBuffer         As W5500_RX.ReleaseRXBuffer,
    ExtractRXBuffer         As W5500_RX.ExtractRXBuffer,
    SeekToken               As W5500_RX.SeekToken,
    MoveRXBufferPtr         As W5500_RX.MoveRXBufferPtr,
    GetRXByte               As W5500_RX.GetRXByte,
    SeekRXByte              As W5500_RX.SeekRXByte, 
    WriteSocketReg          As W5500_Regs.WriteSocketReg,
    WriteCommonReg          As W5500_Regs.WriteCommonReg,    
    WriteSocketWord         As W5500_Regs.WriteSocketWord,
    WriteCommonWord         As W5500_Regs.WriteCommonWord,
    ReadSocketReg           As W5500_Regs.ReadSocketReg,
    ReadCommonReg           As W5500_Regs.ReadCommonReg,
    ReadSocketWord          As W5500_Regs.ReadSocketWord,
    SARtoString             As W5500_Regs.SARtoString,
    SIPRtoString            As W5500_Regs.SIPRtoString,
    SUBRtoString            As W5500_Regs.SUBRtoString,
    GARtoString             As W5500_Regs.GARtoString,
    SeekSocketBufByte       As W5500_Regs.SeekSocketBufByte,
    GetBase                 As W5500_Consts.GetBase, 
    Socket                  As W5500_Socket.Socket,
    Close                   As W5500_Socket.Close,
    Disconnect              As W5500_Socket.Disconnect,
    Connect                 As W5500_Socket.Connect

  
Private Dim 
    TimeoutCount(MAX_SOCK_NUM)      As Word,
    TimeoutEnabled(MAX_SOCK_NUM)    As Boolean

{
****************************************************************************
* Name    : SockTimeoutEnable                                              *
****************************************************************************
} 
Public Sub SockTimeoutEnable(Sock As Byte)
    If TimeoutEnabled(Sock) = false Then
        TimeoutCount(Sock) = 0 
        TimeoutEnabled(Sock) = true   
    End If
End Sub

{
****************************************************************************
* Name    : SockTimeout                                                    *
****************************************************************************
} 
Public Function SockTimeout(Sock As Byte) As Boolean
    result = false
    If TimeoutEnabled(Sock) Then    
        If TimeoutCount(Sock) > cSOCK_TIMEOUT Then
            TimeoutCount(Sock) = 0
            TimeoutEnabled(Sock) = false
            result = true
        Else
            TimeoutCount(Sock) = TimeoutCount(Sock) + 1
        End If
    End If
End Function  

{
****************************************************************************
* Name    : SockTimeoutDisable                                             *
****************************************************************************
} 
Public Sub SockTimeoutDisable(Sock As Byte)
    TimeoutCount(Sock) = 0
    TimeoutEnabled(Sock) = false
End Sub
    
{
****************************************************************************
* Name    : SetPing                                                        *
****************************************************************************
} 
Public Sub SetPing(Enabled As Boolean)
    If Enabled Then
        WriteCommonReg(rMR,ReadCommonReg(rMR) And %11101111)
    Else
        WriteCommonReg(rMR,ReadCommonReg(rMR) Or %00010000)
    End If    
End Sub

{
****************************************************************************
* Name    : SetWOL                                                         *
****************************************************************************
} 
Public Sub SetWOL(Enabled As Boolean)
    If Enabled Then
        WriteCommonReg(rMR,ReadCommonReg(rMR) And %11011111)
    Else
        WriteCommonReg(rMR,ReadCommonReg(rMR) Or %00100000)
    End If    
End Sub

{
****************************************************************************
* Name    : SetPPOE                                                        *
****************************************************************************
} 
Public Sub SetPPOE(Enabled As Boolean)
    If Enabled Then
        WriteCommonReg(rMR,ReadCommonReg(rMR) And %11110111)
    Else
        WriteCommonReg(rMR,ReadCommonReg(rMR) Or %00001000)
    End If    
End Sub

{
****************************************************************************
* Name    : SetARP                                                         *
****************************************************************************
} 
Public Sub SetARP(Enabled As Boolean)
    If Enabled Then
        WriteCommonReg(rMR,ReadCommonReg(rMR) And %11111101)
    Else
        WriteCommonReg(rMR,ReadCommonReg(rMR) Or %00000010)
    End If    
End Sub


{
****************************************************************************
* Name    : Listen                                                         *
****************************************************************************
} 
Public Function Listen(sock As Byte) As Boolean
    Result = false
    If ReadSocketReg(sock, rSn_SR) = cSOCK_INIT Then
        WriteSocketReg(sock, rSn_CR, cSn_CR_LISTEN)
        // Attendo la fine del processo di Listening
        While(ReadSocketReg(sock, rSn_CR) = 1 )
        End While

        // Verifico se  nello stato di Listen 
        If ReadSocketReg(sock, rSn_SR) = cSOCK_LISTEN Then
            Result = true
        Else
            Close(sock)
        End If
    End If
End Function


Dim i As Byte

For i = 0 To MAX_SOCK_NUM - 1
    TimeoutCount(i) = 0
    TimeoutEnabled(i) = false
Next 

